package javawrapper;

import java.util.Vector;

/**
 * This rolls up a set of cell coordinates into a defined subplot.
 * <p>Copyright: Copyright (c) Charles D. Canham 2003</p>
 * <p>Company: Institute of Ecosystem Studies</p>
 * @author Lora E. Murphy
 * @version 1.0
 *
 * <br>Edit history:
 * <br>------------------
 * <br>April 28, 2004: Submitted in beta version (LEM)
 */
class Subplot {
  private String m_sName; /**<Subplot name.*/
  private Vector mp_oCells; /**<List of cells that make up the subplot.*/

  /**
   * Constructor
   * @param sName Name of the subplot
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public Subplot(String sName) {
    mp_oCells = new Vector(0);
    m_sName = sName;
  }

  /**
   * Adds a grid cell to this subplot.  If it is a duplicate, it is not added.
   * @param iX X grid cell coordinate
   * @param iY Y grid cell coordinate
   * @param oPlot Plot object
   * @throws ModelException if the grid cell coordinate is invalid
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public void AddCell(int iX, int iY, Plot oPlot) throws ModelException {
    Cell oCell;
    int i;

    //Check to see if this grid cell already exists
    for (i = 0; i < mp_oCells.size(); i++) {
      oCell = (Cell) mp_oCells.elementAt(i);
      if (iX == oCell.GetX() && iY == oCell.GetY()) {
        return;
      }
    }

    //Add it
    oCell = new Cell(iX, iY, oPlot);
    mp_oCells.add(oCell);
  }

  /**
   * Adds a grid cell to this subplot.  If it is a duplicate, it is not added.
   * @param oCell Cell to add.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public void AddCell(Cell oCell) {
    Cell oExistingCell;
    int i;

    //Check to see if this grid cell already exists
    for (i = 0; i < mp_oCells.size(); i++) {
      oExistingCell = (Cell) mp_oCells.elementAt(i);
      if (oExistingCell.GetX() == oCell.GetX() &&
          oExistingCell.GetY() == oCell.GetY()) {
        return;
      }
    }

    //Add it
    mp_oCells.add(oCell);

  }

  /**
   * Gets the number of cells in this subplot.
   * @return The number of cells.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public int GetNumberOfCells() {
    return mp_oCells.size();
  }

  /**
   * Gets the subplot's name.
   * @return The subplot's name.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public String GetSubplotName() {
    return m_sName;
  }

  /**
   * Gets the cell at a particular index.  If the index is invalid, returns
   * null.
   * @param iIndex Index of the cell.
   * @return Cell.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public Cell GetCell(int iIndex) {
    if (iIndex < 0 || iIndex >= mp_oCells.size()) {
      return null;
    }

    return (Cell) mp_oCells.elementAt(iIndex);
  }
}
