package javawrapper;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.io.File;

/**
 * Window used by the user to set up summary (short) output files.
 * <p>Copyright: Copyright (c) 2003 Charles D. Canham</p>
 * <p>Company: Institute of Ecosystem Studies</p>
 * @author Lora E. Murphy
 * @version 1.0
 *
 * <br>Edit history:
 * <br>------------------
 * <br>April 28, 2004: Submitted in beta version (LEM)
 */

public class SummaryOutputFileSetup
    extends JDialog
    implements ActionListener {
  OutputBehaviors m_oOutputData;
  MainWindow m_oWindow;

  private String m_sHelpID = "windows.summary_output";

  JTextField m_jSummaryOutputFileName = new JTextField();
  JCheckBox m_jSeedlingAbsoluteDensityCheck = new JCheckBox();
  JCheckBox m_jSaplingRelativeBACheck = new JCheckBox();
  JCheckBox m_jSaplingAbsoluteBACheck = new JCheckBox();
  JCheckBox m_jSaplingRelativeDensityCheck = new JCheckBox();
  JCheckBox m_jSaplingAbsoluteDensityCheck = new JCheckBox();
  JCheckBox m_jAdultRelativeBACheck = new JCheckBox();
  JCheckBox m_jAdultAbsoluteBACheck = new JCheckBox();
  JCheckBox m_jAdultRelativeDensityCheck = new JCheckBox();
  JCheckBox m_jAdultAbsoluteDensityCheck = new JCheckBox();
  JCheckBox m_jSnagRelativeBACheck = new JCheckBox();
  JCheckBox m_jSnagAbsoluteBACheck = new JCheckBox();
  JCheckBox m_jSnagRelativeDensityCheck = new JCheckBox();
  JCheckBox m_jSnagAbsoluteDensityCheck = new JCheckBox();

  /**
   * Constructor.
   * @param oParent Parent window in which to display this dialog.
   * @param oData OutputBehaviors object.
   * @param oWindow MainWindow window.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public SummaryOutputFileSetup(JDialog oParent, OutputBehaviors oData,
                                MainWindow oWindow) {
    super(oParent, "Summary Output File Setup", true);

    //Help ID
    oWindow.m_oHelpBroker.enableHelpKey(this.getRootPane(), m_sHelpID, null);

    try {
      m_oOutputData = oData;
      m_oWindow = oWindow;
      jbInit();
      LoadData();
    }
    catch (Exception e) {
      e.printStackTrace();
    }
  }

  public void actionPerformed(ActionEvent oEvent) {
    if (oEvent.getActionCommand().equals("OK")) {
      try {
        //Set the file name
        m_oOutputData.SetShortOutputFileName(m_jSummaryOutputFileName.
                                             getText());

        //Set seedling density
        if (m_jSeedlingAbsoluteDensityCheck.isSelected()) {
          m_oOutputData.SetSaveAbsoluteDensity(TreePopulation.SEEDLING, true);
        }
        else {
          m_oOutputData.SetSaveAbsoluteDensity(TreePopulation.SEEDLING, false);
        }

        //Set sapling absolute density
        if (m_jSaplingAbsoluteDensityCheck.isSelected()) {
          m_oOutputData.SetSaveAbsoluteDensity(TreePopulation.SAPLING, true);
        }
        else {
          m_oOutputData.SetSaveAbsoluteDensity(TreePopulation.SAPLING, false);
        }

        //Set sapling relative density
        if (m_jSaplingRelativeDensityCheck.isSelected()) {
          m_oOutputData.SetSaveRelativeDensity(TreePopulation.SAPLING, true);
        }
        else {
          m_oOutputData.SetSaveRelativeDensity(TreePopulation.SAPLING, false);
        }

        //Set sapling absolute basal area
        if (m_jSaplingAbsoluteBACheck.isSelected()) {
          m_oOutputData.SetSaveAbsoluteBasalArea(TreePopulation.SAPLING, true);
        }
        else {
          m_oOutputData.SetSaveAbsoluteBasalArea(TreePopulation.SAPLING, false);
        }

        //Set sapling relative basal area
        if (m_jSaplingRelativeBACheck.isSelected()) {
          m_oOutputData.SetSaveRelativeBasalArea(TreePopulation.SAPLING, true);
        }
        else {
          m_oOutputData.SetSaveRelativeBasalArea(TreePopulation.SAPLING, false);
        }

        //Set adult absolute density
        if (m_jAdultAbsoluteDensityCheck.isSelected()) {
          m_oOutputData.SetSaveAbsoluteDensity(TreePopulation.ADULT, true);
        }
        else {
          m_oOutputData.SetSaveAbsoluteDensity(TreePopulation.ADULT, false);
        }

        //Set adult relative density
        if (m_jAdultRelativeDensityCheck.isSelected()) {
          m_oOutputData.SetSaveRelativeDensity(TreePopulation.ADULT, true);
        }
        else {
          m_oOutputData.SetSaveRelativeDensity(TreePopulation.ADULT, false);
        }

        //Set adult absolute basal area
        if (m_jAdultAbsoluteBACheck.isSelected()) {
          m_oOutputData.SetSaveAbsoluteBasalArea(TreePopulation.ADULT, true);
        }
        else {
          m_oOutputData.SetSaveAbsoluteBasalArea(TreePopulation.ADULT, false);
        }

        //Set adult relative basal area
        if (m_jAdultRelativeBACheck.isSelected()) {
          m_oOutputData.SetSaveRelativeBasalArea(TreePopulation.ADULT, true);
        }
        else {
          m_oOutputData.SetSaveRelativeBasalArea(TreePopulation.ADULT, false);
        }

        //Set snag absolute density
        if (m_jSnagAbsoluteDensityCheck.isSelected()) {
          m_oOutputData.SetSaveAbsoluteDensity(TreePopulation.SNAG, true);
        }
        else {
          m_oOutputData.SetSaveAbsoluteDensity(TreePopulation.SNAG, false);
        }

        //Set snag relative density
        if (m_jSnagRelativeDensityCheck.isSelected()) {
          m_oOutputData.SetSaveRelativeDensity(TreePopulation.SNAG, true);
        }
        else {
          m_oOutputData.SetSaveRelativeDensity(TreePopulation.SNAG, false);
        }

        //Set snag absolute basal area
        if (m_jSnagAbsoluteBACheck.isSelected()) {
          m_oOutputData.SetSaveAbsoluteBasalArea(TreePopulation.SNAG, true);
        }
        else {
          m_oOutputData.SetSaveAbsoluteBasalArea(TreePopulation.SNAG, false);
        }

        //Set snag relative basal area
        if (m_jSnagRelativeBACheck.isSelected()) {
          m_oOutputData.SetSaveRelativeBasalArea(TreePopulation.SNAG, true);
        }
        else {
          m_oOutputData.SetSaveRelativeBasalArea(TreePopulation.SNAG, false);
        }

        // Short output
        Behavior oBeh = m_oOutputData.GetBehaviorByKey("short output");
        m_oOutputData.UpdateIsEnabled(oBeh);
        m_oOutputData.ValidateData(null);

        //Close window
        this.setVisible(false);
        this.dispose();
      }
      catch (ModelException oErr) {
        ErrorGUI oHandler = new ErrorGUI(this);
        oHandler.WriteErrorMessage(oErr);
      }
    }
    else if (oEvent.getActionCommand().equals("Cancel")) {
      //Close window
      this.setVisible(false);
      this.dispose();
    }
    else if (oEvent.getActionCommand().equals("EditSubplots")) {
      try {
        SubplotEdit oWindow = new SubplotEdit(this, m_oOutputData,
                                              m_oOutputData.m_oManager.
                                              GetDisturbanceBehaviors(),
                                              m_oOutputData.m_oManager.
                                              GetPlantingBehaviors());
        oWindow.pack();

        //Make sure the window's not too big - make it fit within the
        //bounds of the main application window
        if (oWindow.getSize().height > m_oWindow.getSize().height ||
            oWindow.getSize().width > m_oWindow.getSize().width) {

          int iWidth = java.lang.Math.min(oWindow.getSize().width,
                                          m_oWindow.getSize().width - 10);
          int iHeight = java.lang.Math.min(oWindow.getSize().height,
                                           m_oWindow.getSize().height - 10);

          oWindow.setBounds(oWindow.getBounds().x, oWindow.getBounds().y,
                            iWidth, iHeight);
        }

        oWindow.setLocationRelativeTo(null);
        oWindow.setVisible(true);
      }
      catch (ModelException oErr) {
        ErrorGUI oHandler = new ErrorGUI(this);
        oHandler.WriteErrorMessage(oErr);
      }
    }
    else if (oEvent.getActionCommand().equals("Browse")) {
      JFileChooser jChooser = new JFileChooser();

      //If there's a value for the detailed output file, navigate to that directory
      File oDir = new File(m_jSummaryOutputFileName.getText());
      if (oDir.getParentFile() != null && oDir.getParentFile().exists()) {
        jChooser.setCurrentDirectory(oDir.getParentFile());
      }
      else {
        //No value for file - navigate to parameter file directory
        jChooser.setCurrentDirectory(m_oWindow.m_oFileDirectory);
      }

      jChooser.setSelectedFile(new File(oDir.getName()));
      jChooser.setFileFilter(new OutFileFilter());

      int returnVal = jChooser.showSaveDialog(this);
      if (returnVal == JFileChooser.APPROVE_OPTION) {
        //User chose a file - does it already exist?
        File oFile = jChooser.getSelectedFile();

        //Does this file already exist?
        if (oFile.exists()) {
          returnVal = JOptionPane.showConfirmDialog(this,
                                             "This file already exists.  Do you wish to overwrite it?",
                                             "Model",
                                             JOptionPane.YES_NO_OPTION);
          if (returnVal == JOptionPane.YES_OPTION) {
            String sFileName = oFile.getAbsolutePath();
            m_jSummaryOutputFileName.setText(sFileName);
          }
        }
        else {
          //File didn't exist
          String sFileName = oFile.getAbsolutePath();
          m_jSummaryOutputFileName.setText(sFileName);
        }
      }
    }
  }

  /**
   * This loads data from the output behavior into the window.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   * <br>June 24, 2004:  Added snag support (LEM)
   */
  private void LoadData() {
    try {
      //Filename
      if (m_oOutputData.GetShortOutputFileName().length() > 0) {
        m_jSummaryOutputFileName.setText(m_oOutputData.
                                         GetShortOutputFileName());
      }

      //Seedling absolute density
      m_jSeedlingAbsoluteDensityCheck.setSelected(m_oOutputData.
                                                  GetSaveAbsoluteDensity(
          TreePopulation.SEEDLING));

      //Sapling absolute density
      m_jSaplingAbsoluteDensityCheck.setSelected(m_oOutputData.
                                                 GetSaveAbsoluteDensity(
          TreePopulation.SAPLING));

      //Sapling relative density
      m_jSaplingRelativeDensityCheck.setSelected(m_oOutputData.
                                                 GetSaveRelativeDensity(
          TreePopulation.SAPLING));

      //Sapling absolute basal area
      m_jSaplingAbsoluteBACheck.setSelected(m_oOutputData.
                                            GetSaveAbsoluteBasalArea(
          TreePopulation.SAPLING));

      //Sapling relative basal area
      m_jSaplingRelativeBACheck.setSelected(m_oOutputData.
                                            GetSaveRelativeBasalArea(
          TreePopulation.SAPLING));

      //Adult absolute density
      m_jAdultAbsoluteDensityCheck.setSelected(m_oOutputData.
                                               GetSaveAbsoluteDensity(
          TreePopulation.ADULT));

      //Adult relative density
      m_jAdultRelativeDensityCheck.setSelected(m_oOutputData.
                                               GetSaveRelativeDensity(
          TreePopulation.ADULT));

      //Adult absolute basal area
      m_jAdultAbsoluteBACheck.setSelected(m_oOutputData.
                                          GetSaveAbsoluteBasalArea(
          TreePopulation.ADULT));

      //Adult relative basal area
      m_jAdultRelativeBACheck.setSelected(m_oOutputData.
                                          GetSaveRelativeBasalArea(
          TreePopulation.ADULT));

      //Snag absolute density
      m_jSnagAbsoluteDensityCheck.setSelected(m_oOutputData.
                                              GetSaveAbsoluteDensity(
          TreePopulation.SNAG));

      //Snag relative density
      m_jSnagRelativeDensityCheck.setSelected(m_oOutputData.
                                              GetSaveRelativeDensity(
          TreePopulation.SNAG));

      //Snag absolute basal area
      m_jSnagAbsoluteBACheck.setSelected(m_oOutputData.
                                         GetSaveAbsoluteBasalArea(
          TreePopulation.SNAG));

      //Snag relative basal area
      m_jSnagRelativeBACheck.setSelected(m_oOutputData.
                                         GetSaveRelativeBasalArea(
          TreePopulation.SNAG));

    }
    catch (ModelException oErr) {
      ErrorGUI oHandler = new ErrorGUI(this);
      oHandler.WriteErrorMessage(oErr);
    }
  }

  private void jbInit() throws Exception {

    JPanel jMainPanel = new JPanel();
    jMainPanel.setLayout(new BoxLayout(jMainPanel, BoxLayout.PAGE_AXIS));

    //Label at the top
    JLabel jTempLabel = new JLabel("Set Up Summary Output File...");
    jTempLabel.setFont(new ModelFont(java.awt.Font.BOLD, 2));
    jTempLabel.setHorizontalAlignment(SwingConstants.LEFT);
    jTempLabel.setAlignmentX(java.awt.Component.LEFT_ALIGNMENT);
    jMainPanel.add(jTempLabel);

    //Filename and browse button
    jTempLabel = new JLabel("Output File Name:");
    jTempLabel.setFont(new ModelFont());
    jTempLabel.setHorizontalAlignment(SwingConstants.LEFT);
    jTempLabel.setAlignmentX(java.awt.Component.LEFT_ALIGNMENT);
    JPanel jTempPanel = new JPanel(new FlowLayout(FlowLayout.LEFT));
    jTempPanel.setAlignmentX(java.awt.Component.LEFT_ALIGNMENT);
    jTempPanel.setBorder(BorderFactory.createEmptyBorder(10, 0, 0, 0));
    jTempPanel.add(jTempLabel);
    jMainPanel.add(jTempPanel);

    m_jSummaryOutputFileName.setFont(new ModelFont());
    m_jSummaryOutputFileName.setText("");
    m_jSummaryOutputFileName.setPreferredSize(new Dimension(300,
        (int) m_jSummaryOutputFileName.getPreferredSize().getHeight()));
    JButton jBrowseButton = new JButton("Browse");
    jBrowseButton.setFont(new ModelFont());
    jBrowseButton.addActionListener(this);
    jBrowseButton.setActionCommand("Browse");
    jBrowseButton.setAlignmentX(java.awt.Component.LEFT_ALIGNMENT);
    jTempPanel = new JPanel();
    jTempPanel.add(m_jSummaryOutputFileName);
    jTempPanel.add(jBrowseButton);
    jTempPanel.setAlignmentX(java.awt.Component.LEFT_ALIGNMENT);
    jMainPanel.add(jTempPanel);

    //Panel for seedlings
    jTempLabel = new JLabel("Seedlings");
    jTempLabel.setFont(new ModelFont(java.awt.Font.BOLD));
    jTempLabel.setAlignmentX(java.awt.Component.LEFT_ALIGNMENT);
    m_jSeedlingAbsoluteDensityCheck.setBackground(SystemColor.control);
    m_jSeedlingAbsoluteDensityCheck.setFont(new ModelFont());
    m_jSeedlingAbsoluteDensityCheck.setText("Absolute Density");
    m_jSeedlingAbsoluteDensityCheck.setAlignmentX(java.awt.Component.
                                                  LEFT_ALIGNMENT);
    jTempPanel = new JPanel();
    jTempPanel.setLayout(new BoxLayout(jTempPanel, BoxLayout.PAGE_AXIS));
    jTempPanel.setAlignmentX(java.awt.Component.LEFT_ALIGNMENT);
    jTempPanel.add(jTempLabel);
    jTempPanel.add(m_jSeedlingAbsoluteDensityCheck);
    jMainPanel.add(jTempPanel);

    //Panel for sapling and adult options
    jTempLabel = new JLabel("Saplings");
    jTempLabel.setFont(new ModelFont(java.awt.Font.BOLD));
    JLabel jTempLabel2 = new JLabel("Adults");
    jTempLabel2.setFont(new ModelFont(java.awt.Font.BOLD));
    JLabel jTempLabel3 = new JLabel("Snags");
    jTempLabel3.setFont(new ModelFont(java.awt.Font.BOLD));

    m_jSaplingRelativeBACheck.setBackground(SystemColor.control);
    m_jSaplingRelativeBACheck.setFont(new ModelFont());
    m_jSaplingRelativeBACheck.setText("Relative Basal Area");
    m_jSaplingAbsoluteBACheck.setBackground(SystemColor.control);
    m_jSaplingAbsoluteBACheck.setFont(new ModelFont());
    m_jSaplingAbsoluteBACheck.setText("Absolute Basal Area");
    m_jSaplingRelativeDensityCheck.setBackground(SystemColor.control);
    m_jSaplingRelativeDensityCheck.setFont(new ModelFont());
    m_jSaplingRelativeDensityCheck.setText("Relative Density");
    m_jSaplingAbsoluteDensityCheck.setBackground(SystemColor.control);
    m_jSaplingAbsoluteDensityCheck.setFont(new ModelFont());
    m_jSaplingAbsoluteDensityCheck.setActionCommand("Absolute Density");
    m_jSaplingAbsoluteDensityCheck.setSelected(false);
    m_jSaplingAbsoluteDensityCheck.setText("Absolute Density");

    m_jAdultRelativeBACheck.setBackground(SystemColor.control);
    m_jAdultRelativeBACheck.setFont(new ModelFont());
    m_jAdultRelativeBACheck.setText("Relative Basal Area");
    m_jAdultAbsoluteBACheck.setBackground(SystemColor.control);
    m_jAdultAbsoluteBACheck.setFont(new ModelFont());
    m_jAdultAbsoluteBACheck.setText("Absolute Basal Area");
    m_jAdultRelativeDensityCheck.setBackground(SystemColor.control);
    m_jAdultRelativeDensityCheck.setFont(new ModelFont());
    m_jAdultRelativeDensityCheck.setText("Relative Density");
    m_jAdultAbsoluteDensityCheck.setBackground(SystemColor.control);
    m_jAdultAbsoluteDensityCheck.setFont(new ModelFont());
    m_jAdultAbsoluteDensityCheck.setText("Absolute Density");

    m_jSnagRelativeBACheck.setBackground(SystemColor.control);
    m_jSnagRelativeBACheck.setFont(new ModelFont());
    m_jSnagRelativeBACheck.setText("Relative Basal Area");
    m_jSnagAbsoluteBACheck.setBackground(SystemColor.control);
    m_jSnagAbsoluteBACheck.setFont(new ModelFont());
    m_jSnagAbsoluteBACheck.setText("Absolute Basal Area");
    m_jSnagRelativeDensityCheck.setBackground(SystemColor.control);
    m_jSnagRelativeDensityCheck.setFont(new ModelFont());
    m_jSnagRelativeDensityCheck.setText("Relative Density");
    m_jSnagAbsoluteDensityCheck.setBackground(SystemColor.control);
    m_jSnagAbsoluteDensityCheck.setFont(new ModelFont());
    m_jSnagAbsoluteDensityCheck.setText("Absolute Density");

    jTempPanel = new JPanel(new GridLayout(0, 3));
    jTempPanel.setAlignmentX(java.awt.Component.LEFT_ALIGNMENT);
    jTempPanel.add(jTempLabel);
    jTempPanel.add(jTempLabel2);
    jTempPanel.add(jTempLabel3);
    jTempPanel.add(m_jSaplingRelativeBACheck);
    jTempPanel.add(m_jAdultRelativeBACheck);
    jTempPanel.add(m_jSnagRelativeBACheck);
    jTempPanel.add(m_jSaplingAbsoluteBACheck);
    jTempPanel.add(m_jAdultAbsoluteBACheck);
    jTempPanel.add(m_jSnagAbsoluteBACheck);
    jTempPanel.add(m_jSaplingRelativeDensityCheck);
    jTempPanel.add(m_jAdultRelativeDensityCheck);
    jTempPanel.add(m_jSnagRelativeDensityCheck);
    jTempPanel.add(m_jSaplingAbsoluteDensityCheck);
    jTempPanel.add(m_jAdultAbsoluteDensityCheck);
    jTempPanel.add(m_jSnagAbsoluteDensityCheck);
    jMainPanel.add(jTempPanel);

    //Button to set up subplots
    JButton jSubplotButton = new JButton("Set up subplots...");
    jSubplotButton.setFont(new ModelFont());
    jSubplotButton.setActionCommand("EditSubplots");
    jSubplotButton.addActionListener(this);
    jSubplotButton.setAlignmentX(java.awt.Component.LEFT_ALIGNMENT);
    jMainPanel.add(jSubplotButton);

    jMainPanel.setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));

    this.getContentPane().setLayout(new BorderLayout());
    this.getContentPane().add(jMainPanel, BorderLayout.CENTER);

    this.getContentPane().add(new OKCancelButtonPanel(this,
        m_oWindow.m_oHelpBroker, m_sHelpID), BorderLayout.SOUTH);

  }
}
