package javawrapper;

import junit.framework.*;

/**
 * Tests the Cell class.
 * <p>Copyright: Copyright (c) Charles D. Canham 2003</p>
 * <p>Company: Institute of Ecosystem Studies</p>
 * @author Lora E. Murphy
 * @version 1.0
 *
 * <br>Edit history:
 * <br>------------------
 * <br>April 28, 2004: Submitted in beta version (LEM)
 */
public class TestCell
    extends TestCase {

  public TestCell(String name) {
    super(name);
  }

  protected void setUp() throws Exception {
    super.setUp();
  }

  protected void tearDown() throws Exception {
    super.tearDown();
  }

  public void testTransformOldGridCell1() {
    try {
      float fOldXGridLength = 8;
      float fOldYGridLength = 8;
      int iNumOldXGrids = 25;
      int iNumOldYGrids = 25;
      Plot oPlot = new Plot(null);
      oPlot.SetPlotXLength(200);
      oPlot.SetPlotYLength(200);

      Cell[] p_oActualReturn;

      //---------------------------------------------------------------------
      //Case:  Old cell lengths equal 8.
      //---------------------------------------------------------------------
      //Transform and add a list of old cells
      p_oActualReturn = Cell.TransformOldGridCell(0, fOldXGridLength,
                                                  fOldYGridLength,
                                                  iNumOldXGrids, iNumOldYGrids,
                                                  oPlot);
      assertEquals(1, p_oActualReturn.length);
      assertEquals(0, p_oActualReturn[0].GetX());
      assertEquals(0, p_oActualReturn[0].GetY());

      p_oActualReturn = Cell.TransformOldGridCell(1, fOldXGridLength,
                                                  fOldYGridLength,
                                                  iNumOldXGrids, iNumOldYGrids,
                                                  oPlot);
      assertEquals(1, p_oActualReturn.length);
      assertEquals(1, p_oActualReturn[0].GetX());
      assertEquals(0, p_oActualReturn[0].GetY());

      p_oActualReturn = Cell.TransformOldGridCell(2, fOldXGridLength,
                                                  fOldYGridLength,
                                                  iNumOldXGrids, iNumOldYGrids,
                                                  oPlot);
      assertEquals(1, p_oActualReturn.length);
      assertEquals(2, p_oActualReturn[0].GetX());
      assertEquals(0, p_oActualReturn[0].GetY());

      p_oActualReturn = Cell.TransformOldGridCell(3, fOldXGridLength,
                                                  fOldYGridLength,
                                                  iNumOldXGrids, iNumOldYGrids,
                                                  oPlot);
      assertEquals(1, p_oActualReturn.length);
      assertEquals(3, p_oActualReturn[0].GetX());
      assertEquals(0, p_oActualReturn[0].GetY());

      p_oActualReturn = Cell.TransformOldGridCell(4, fOldXGridLength,
                                                  fOldYGridLength,
                                                  iNumOldXGrids, iNumOldYGrids,
                                                  oPlot);
      assertEquals(1, p_oActualReturn.length);
      assertEquals(4, p_oActualReturn[0].GetX());
      assertEquals(0, p_oActualReturn[0].GetY());

      p_oActualReturn = Cell.TransformOldGridCell(5, fOldXGridLength,
                                                  fOldYGridLength,
                                                  iNumOldXGrids, iNumOldYGrids,
                                                  oPlot);
      assertEquals(1, p_oActualReturn.length);
      assertEquals(5, p_oActualReturn[0].GetX());
      assertEquals(0, p_oActualReturn[0].GetY());

      p_oActualReturn = Cell.TransformOldGridCell(50, fOldXGridLength,
                                                  fOldYGridLength,
                                                  iNumOldXGrids, iNumOldYGrids,
                                                  oPlot);
      assertEquals(1, p_oActualReturn.length);
      assertEquals(0, p_oActualReturn[0].GetX());
      assertEquals(2, p_oActualReturn[0].GetY());

      p_oActualReturn = Cell.TransformOldGridCell(58, fOldXGridLength,
                                                  fOldYGridLength,
                                                  iNumOldXGrids, iNumOldYGrids,
                                                  oPlot);
      assertEquals(1, p_oActualReturn.length);
      assertEquals(8, p_oActualReturn[0].GetX());
      assertEquals(2, p_oActualReturn[0].GetY());

      p_oActualReturn = Cell.TransformOldGridCell(64, fOldXGridLength,
                                                  fOldYGridLength,
                                                  iNumOldXGrids, iNumOldYGrids,
                                                  oPlot);
      assertEquals(1, p_oActualReturn.length);
      assertEquals(14, p_oActualReturn[0].GetX());
      assertEquals(2, p_oActualReturn[0].GetY());

      p_oActualReturn = Cell.TransformOldGridCell(624, fOldXGridLength,
                                                  fOldYGridLength,
                                                  iNumOldXGrids, iNumOldYGrids,
                                                  oPlot);
      assertEquals(1, p_oActualReturn.length);
      assertEquals(24, p_oActualReturn[0].GetX());
      assertEquals(24, p_oActualReturn[0].GetY());

    }
    catch (ModelException oErr) {
      fail("Normal processing for TransformOldGridCell failed with message " +
           oErr.sMessage);
    }
  }

  public void testTransformOldGridCell2() {
    try {
      float fOldXGridLength = 6;
      float fOldYGridLength = 5;
      int iNumOldXGrids = 20;
      int iNumOldYGrids = 20;
      Plot oPlot = new Plot(null);
      HarvestData oHarvestData = new HarvestData(); //let this collect cells
      oPlot.SetPlotXLength(100);
      oPlot.SetPlotYLength(120);
      int[] p_iOldCells = {
          0, 1, 2, 3, 4, 5, 50, 58, 64, 399};
      int i, j;

      Cell[] p_oActualReturn;
      Cell oCell;

      //---------------------------------------------------------------------
      //Case:  Old cell lengths less than 8.
      //---------------------------------------------------------------------
      //Transform and add a list of old cells
      for (i = 0; i < p_iOldCells.length; i++) {
        p_oActualReturn = Cell.TransformOldGridCell(p_iOldCells[i],
            fOldXGridLength,
            fOldYGridLength,
            iNumOldXGrids, iNumOldYGrids,
            oPlot);
        for (j = 0; j < p_oActualReturn.length; j++) {
          oHarvestData.AddCell(p_oActualReturn[j]);
        }
      }

      assertEquals(8, oHarvestData.GetNumberOfCells());
      oCell = oHarvestData.GetCell(0);
      assertEquals(0, oCell.GetX());
      assertEquals(0, oCell.GetY());

      oCell = oHarvestData.GetCell(1);
      assertEquals(1, oCell.GetX());
      assertEquals(0, oCell.GetY());

      oCell = oHarvestData.GetCell(2);
      assertEquals(2, oCell.GetX());
      assertEquals(0, oCell.GetY());

      oCell = oHarvestData.GetCell(3);
      assertEquals(3, oCell.GetX());
      assertEquals(0, oCell.GetY());

      oCell = oHarvestData.GetCell(4);
      assertEquals(6, oCell.GetX());
      assertEquals(1, oCell.GetY());

      oCell = oHarvestData.GetCell(5);
      assertEquals(11, oCell.GetX());
      assertEquals(1, oCell.GetY());

      oCell = oHarvestData.GetCell(6);
      assertEquals(2, oCell.GetX());
      assertEquals(2, oCell.GetY());

      oCell = oHarvestData.GetCell(7);
      assertEquals(11, oCell.GetX());
      assertEquals(14, oCell.GetY());

    }
    catch (ModelException oErr) {
      fail("Normal processing for TransformOldGridCell failed with message " +
           oErr.sMessage);
    }
  }

  public void testTransformOldGridCell3() {
    try {
      float fOldXGridLength = 10;
      float fOldYGridLength = 12;
      int iNumOldXGrids = 20;
      int iNumOldYGrids = 20;
      Plot oPlot = new Plot(null);
      HarvestData oHarvestData = new HarvestData(); //let this collect cells
      oPlot.SetPlotXLength(240);
      oPlot.SetPlotYLength(200);
      int[] p_iOldCells = {
          0, 1, 2, 3, 4, 5, 50, 58, 64, 399};
      int i, j;
      boolean bFound;

      Cell[] p_oActualReturn;
      Cell oCell;

      //---------------------------------------------------------------------
      //Case:  Old cell lengths between 8 and 16.
      //---------------------------------------------------------------------
      //Transform and add a list of old cells
      for (i = 0; i < p_iOldCells.length; i++) {
        p_oActualReturn = Cell.TransformOldGridCell(p_iOldCells[i],
            fOldXGridLength,
            fOldYGridLength,
            iNumOldXGrids, iNumOldYGrids,
            oPlot);
        for (j = 0; j < p_oActualReturn.length; j++) {
          oHarvestData.AddCell(p_oActualReturn[j]);
        }
      }

      assertEquals(34, oHarvestData.GetNumberOfCells());

      //Make sure all of the expected cells are on the list
      //1
      bFound = false;
      for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
        oCell = oHarvestData.GetCell(i);
        if (oCell.GetX() == 0 && oCell.GetY() == 0) {
          bFound = true;
          break;
        }
      }
      if (!bFound) {
        fail("Didn't find cell 0,0.");
      }

      //2
      bFound = false;
      for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
        oCell = oHarvestData.GetCell(i);
        if (oCell.GetX() == 1 && oCell.GetY() == 1) {
          bFound = true;
          break;
        }
      }
      if (!bFound) {
        fail("Didn't find cell 1,1.");
      }

      //3
      bFound = false;
      for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
        oCell = oHarvestData.GetCell(i);
        if (oCell.GetX() == 1 && oCell.GetY() == 0) {
          bFound = true;
          break;
        }
      }
      if (!bFound) {
        fail("Didn't find cell 1,0.");
      }

      //4
      bFound = false;
      for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
        oCell = oHarvestData.GetCell(i);
        if (oCell.GetX() == 0 && oCell.GetY() == 1) {
          bFound = true;
          break;
        }
      }
      if (!bFound) {
        fail("Didn't find cell 0,1.");
      }

      //5
      bFound = false;
      for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
        oCell = oHarvestData.GetCell(i);
        if (oCell.GetX() == 2 && oCell.GetY() == 1) {
          bFound = true;
          break;
        }
      }
      if (!bFound) {
        fail("Didn't find cell 2,1.");
      }

      //6
      bFound = false;
      for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
        oCell = oHarvestData.GetCell(i);
        if (oCell.GetX() == 2 && oCell.GetY() == 0) {
          bFound = true;
          break;
        }
      }
      if (!bFound) {
        fail("Didn't find cell 2,0.");
      }

      //7
      bFound = false;
      for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
        oCell = oHarvestData.GetCell(i);
        if (oCell.GetX() == 3 && oCell.GetY() == 0) {
          bFound = true;
          break;
        }
      }
      if (!bFound) {
        fail("Didn't find cell 3,0.");
      }

      //8
      bFound = false;
      for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
        oCell = oHarvestData.GetCell(i);
        if (oCell.GetX() == 4 && oCell.GetY() == 1) {
          bFound = true;
          break;
        }
      }
      if (!bFound) {
        fail("Didn't find cell 4,1.");
      }

      //9
      bFound = false;
      for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
        oCell = oHarvestData.GetCell(i);
        if (oCell.GetX() == 4 && oCell.GetY() == 0) {
          bFound = true;
          break;
        }
      }
      if (!bFound) {
        fail("Didn't find cell 4,0.");
      }

      //10
      bFound = false;
      for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
        oCell = oHarvestData.GetCell(i);
        if (oCell.GetX() == 3 && oCell.GetY() == 1) {
          bFound = true;
          break;
        }
      }
      if (!bFound) {
        fail("Didn't find cell 3,1.");
      }

      //11
      bFound = false;
      for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
        oCell = oHarvestData.GetCell(i);
        if (oCell.GetX() == 5 && oCell.GetY() == 1) {
          bFound = true;
          break;
        }
      }
      if (!bFound) {
        fail("Didn't find cell 5,1.");
      }

      //12
      bFound = false;
      for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
        oCell = oHarvestData.GetCell(i);
        if (oCell.GetX() == 5 && oCell.GetY() == 0) {
          bFound = true;
          break;
        }
      }
      if (!bFound) {
        fail("Didn't find cell 5,0.");
      }

      //13
      bFound = false;
      for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
        oCell = oHarvestData.GetCell(i);
        if (oCell.GetX() == 6 && oCell.GetY() == 0) {
          bFound = true;
          break;
        }
      }
      if (!bFound) {
        fail("Didn't find cell 6,0.");
      }

      //14
      bFound = false;
      for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
        oCell = oHarvestData.GetCell(i);
        if (oCell.GetX() == 7 && oCell.GetY() == 1) {
          bFound = true;
          break;
        }
      }
      if (!bFound) {
        fail("Didn't find cell 7,1.");
      }

      //15
      bFound = false;
      for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
        oCell = oHarvestData.GetCell(i);
        if (oCell.GetX() == 6 && oCell.GetY() == 1) {
          bFound = true;
          break;
        }
      }
      if (!bFound) {
        fail("Didn't find cell 6,1.");
      }

      //16
      bFound = false;
      for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
        oCell = oHarvestData.GetCell(i);
        if (oCell.GetX() == 7 && oCell.GetY() == 0) {
          bFound = true;
          break;
        }
      }
      if (!bFound) {
        fail("Didn't find cell 7,0.");
      }

      //17
      bFound = false;
      for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
        oCell = oHarvestData.GetCell(i);
        if (oCell.GetX() == 8 && oCell.GetY() == 1) {
          bFound = true;
          break;
        }
      }
      if (!bFound) {
        fail("Didn't find cell 8,1.");
      }

      //18
      bFound = false;
      for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
        oCell = oHarvestData.GetCell(i);
        if (oCell.GetX() == 8 && oCell.GetY() == 0) {
          bFound = true;
          break;
        }
      }
      if (!bFound) {
        fail("Didn't find cell 8,0.");
      }

      //19
      bFound = false;
      for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
        oCell = oHarvestData.GetCell(i);
        if (oCell.GetX() == 15 && oCell.GetY() == 2) {
          bFound = true;
          break;
        }
      }
      if (!bFound) {
        fail("Didn't find cell 15,2.");
      }

      //20
      bFound = false;
      for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
        oCell = oHarvestData.GetCell(i);
        if (oCell.GetX() == 16 && oCell.GetY() == 3) {
          bFound = true;
          break;
        }
      }
      if (!bFound) {
        fail("Didn't find cell 16,3.");
      }

      //21
      bFound = false;
      for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
        oCell = oHarvestData.GetCell(i);
        if (oCell.GetX() == 15 && oCell.GetY() == 3) {
          bFound = true;
          break;
        }
      }
      if (!bFound) {
        fail("Didn't find cell 15,3.");
      }

      //22
      bFound = false;
      for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
        oCell = oHarvestData.GetCell(i);
        if (oCell.GetX() == 16 && oCell.GetY() == 2) {
          bFound = true;
          break;
        }
      }
      if (!bFound) {
        fail("Didn't find cell 16,2.");
      }

      //23
      bFound = false;
      for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
        oCell = oHarvestData.GetCell(i);
        if (oCell.GetX() == 27 && oCell.GetY() == 2) {
          bFound = true;
          break;
        }
      }
      if (!bFound) {
        fail("Didn't find cell 27,2.");
      }

      //24
      bFound = false;
      for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
        oCell = oHarvestData.GetCell(i);
        if (oCell.GetX() == 28 && oCell.GetY() == 3) {
          bFound = true;
          break;
        }
      }
      if (!bFound) {
        fail("Didn't find cell 28,3.");
      }

      //25
      bFound = false;
      for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
        oCell = oHarvestData.GetCell(i);
        if (oCell.GetX() == 27 && oCell.GetY() == 3) {
          bFound = true;
          break;
        }
      }
      if (!bFound) {
        fail("Didn't find cell 27,3.");
      }

      //26
      bFound = false;
      for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
        oCell = oHarvestData.GetCell(i);
        if (oCell.GetX() == 28 && oCell.GetY() == 2) {
          bFound = true;
          break;
        }
      }
      if (!bFound) {
        fail("Didn't find cell 28,2.");
      }

      //27
      bFound = false;
      for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
        oCell = oHarvestData.GetCell(i);
        if (oCell.GetX() == 6 && oCell.GetY() == 3) {
          bFound = true;
          break;
        }
      }
      if (!bFound) {
        fail("Didn't find cell 6,3.");
      }

      //28
      bFound = false;
      for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
        oCell = oHarvestData.GetCell(i);
        if (oCell.GetX() == 7 && oCell.GetY() == 4) {
          bFound = true;
          break;
        }
      }
      if (!bFound) {
        fail("Didn't find cell 7,4.");
      }

      //29
      bFound = false;
      for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
        oCell = oHarvestData.GetCell(i);
        if (oCell.GetX() == 6 && oCell.GetY() == 4) {
          bFound = true;
          break;
        }
      }
      if (!bFound) {
        fail("Didn't find cell 6,4.");
      }

      //30
      bFound = false;
      for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
        oCell = oHarvestData.GetCell(i);
        if (oCell.GetX() == 7 && oCell.GetY() == 3) {
          bFound = true;
          break;
        }
      }
      if (!bFound) {
        fail("Didn't find cell 7,3.");
      }

      //31
      bFound = false;
      for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
        oCell = oHarvestData.GetCell(i);
        if (oCell.GetX() == 29 && oCell.GetY() == 24) {
          bFound = true;
          break;
        }
      }
      if (!bFound) {
        fail("Didn't find cell 29,24.");
      }

      //32
      bFound = false;
      for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
        oCell = oHarvestData.GetCell(i);
        if (oCell.GetX() == 29 && oCell.GetY() == 23) {
          bFound = true;
          break;
        }
      }
      if (!bFound) {
        fail("Didn't find cell 29,23.");
      }

      //33
      bFound = false;
      for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
        oCell = oHarvestData.GetCell(i);
        if (oCell.GetX() == 28 && oCell.GetY() == 24) {
          bFound = true;
          break;
        }
      }
      if (!bFound) {
        fail("Didn't find cell 28,24.");
      }

      //34
      bFound = false;
      for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
        oCell = oHarvestData.GetCell(i);
        if (oCell.GetX() == 28 && oCell.GetY() == 23) {
          bFound = true;
          break;
        }
      }
      if (!bFound) {
        fail("Didn't find cell 28,23.");
      }

    }
    catch (ModelException oErr) {
      fail("Normal processing for TransformOldGridCell failed with message " +
           oErr.sMessage);
    }
  }

  public void testTransformOldGridCell4() {
    try {
      float fOldXGridLength = 18;
      float fOldYGridLength = 26;
      int iNumOldXGrids = 20;
      int iNumOldYGrids = 20;
      Plot oPlot = new Plot(null);
      HarvestData oHarvestData = new HarvestData(); //let this collect cells
      oPlot.SetPlotXLength(520);
      oPlot.SetPlotYLength(360);
      int[] p_iOldCells = {
          0, 1, 2, 3, 4, 5, 50, 58, 64, 399};
      int i, j;
      boolean bFound;

      Cell[] p_oActualReturn;
      Cell oCell;

      //---------------------------------------------------------------------
      //Case:  Old cell lengths greater than 16.
      //---------------------------------------------------------------------
      //Transform and add a list of old cells
      for (i = 0; i < p_iOldCells.length; i++) {
        p_oActualReturn = Cell.TransformOldGridCell(p_iOldCells[i],
            fOldXGridLength,
            fOldYGridLength,
            iNumOldXGrids, iNumOldYGrids,
            oPlot);
        for (j = 0; j < p_oActualReturn.length; j++) {
          oHarvestData.AddCell(p_oActualReturn[j]);
        }
      }

      assertEquals(108, oHarvestData.GetNumberOfCells());

      //Make sure all of the expected cells are on the list
      //1
      bFound = false;
      for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
        oCell = oHarvestData.GetCell(i);
        if (oCell.GetX() == 0 && oCell.GetY() == 0) {
          bFound = true;
          break;
        }
      }
      if (!bFound) {
        fail("Didn't find cell 0,0.");
      }

      //2
      bFound = false;
      for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
        oCell = oHarvestData.GetCell(i);
        if (oCell.GetX() == 1 && oCell.GetY() == 1) {
          bFound = true;
          break;
        }
      }
      if (!bFound) {
        fail("Didn't find cell 1,1.");
      }

      //3
      bFound = false;
      for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
        oCell = oHarvestData.GetCell(i);
        if (oCell.GetX() == 1 && oCell.GetY() == 0) {
          bFound = true;
          break;
        }
      }
      if (!bFound) {
        fail("Didn't find cell 1,0.");
      }

      //4
      bFound = false;
      for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
        oCell = oHarvestData.GetCell(i);
        if (oCell.GetX() == 0 && oCell.GetY() == 1) {
          bFound = true;
          break;
        }
      }
      if (!bFound) {
        fail("Didn't find cell 0,1.");
      }

      bFound = false;
      for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
        oCell = oHarvestData.GetCell(i);
        if (oCell.GetX() == 0 && oCell.GetY() == 2) {
          bFound = true;
          break;
        }
      }
      if (!bFound) {
        fail("Didn't find cell 0,2.");
      }

      bFound = false;
      for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
        oCell = oHarvestData.GetCell(i);
        if (oCell.GetX() == 1 && oCell.GetY() == 2) {
          bFound = true;
          break;
        }
      }
      if (!bFound) {
        fail("Didn't find cell 1,2.");
      }

      bFound = false;
      for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
        oCell = oHarvestData.GetCell(i);
        if (oCell.GetX() == 2 && oCell.GetY() == 2) {
          bFound = true;
          break;
        }
      }
      if (!bFound) {
        fail("Didn't find cell 2,2.");
      }

      //5
      bFound = false;
      for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
        oCell = oHarvestData.GetCell(i);
        if (oCell.GetX() == 2 && oCell.GetY() == 1) {
          bFound = true;
          break;
        }
      }
      if (!bFound) {
        fail("Didn't find cell 2,1.");
      }

      //6
      bFound = false;
      for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
        oCell = oHarvestData.GetCell(i);
        if (oCell.GetX() == 2 && oCell.GetY() == 0) {
          bFound = true;
          break;
        }
      }
      if (!bFound) {
        fail("Didn't find cell 2,0.");
      }

      //7
      bFound = false;
      for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
        oCell = oHarvestData.GetCell(i);
        if (oCell.GetX() == 3 && oCell.GetY() == 0) {
          bFound = true;
          break;
        }
      }
      if (!bFound) {
        fail("Didn't find cell 3,0.");
      }

      //8
      bFound = false;
      for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
        oCell = oHarvestData.GetCell(i);
        if (oCell.GetX() == 4 && oCell.GetY() == 1) {
          bFound = true;
          break;
        }
      }
      if (!bFound) {
        fail("Didn't find cell 4,1.");
      }

      //9
      bFound = false;
      for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
        oCell = oHarvestData.GetCell(i);
        if (oCell.GetX() == 4 && oCell.GetY() == 0) {
          bFound = true;
          break;
        }
      }
      if (!bFound) {
        fail("Didn't find cell 4,0.");
      }

      bFound = false;
      for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
        oCell = oHarvestData.GetCell(i);
        if (oCell.GetX() == 4 && oCell.GetY() == 2) {
          bFound = true;
          break;
        }
      }
      if (!bFound) {
        fail("Didn't find cell 4,2.");
      }

      //10
      bFound = false;
      for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
        oCell = oHarvestData.GetCell(i);
        if (oCell.GetX() == 3 && oCell.GetY() == 1) {
          bFound = true;
          break;
        }
      }
      if (!bFound) {
        fail("Didn't find cell 3,1.");
      }

      bFound = false;
      for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
        oCell = oHarvestData.GetCell(i);
        if (oCell.GetX() == 3 && oCell.GetY() == 2) {
          bFound = true;
          break;
        }
      }
      if (!bFound) {
        fail("Didn't find cell 3,2.");
      }

      //11
      bFound = false;
      for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
        oCell = oHarvestData.GetCell(i);
        if (oCell.GetX() == 5 && oCell.GetY() == 1) {
          bFound = true;
          break;
        }
      }
      if (!bFound) {
        fail("Didn't find cell 5,1.");
      }

      //12
      bFound = false;
      for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
        oCell = oHarvestData.GetCell(i);
        if (oCell.GetX() == 5 && oCell.GetY() == 0) {
          bFound = true;
          break;
        }
      }
      if (!bFound) {
        fail("Didn't find cell 5,0.");
      }

      bFound = false;
      for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
        oCell = oHarvestData.GetCell(i);
        if (oCell.GetX() == 5 && oCell.GetY() == 2) {
          bFound = true;
          break;
        }
      }
      if (!bFound) {
        fail("Didn't find cell 5,2.");
      }

      //13
      bFound = false;
      for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
        oCell = oHarvestData.GetCell(i);
        if (oCell.GetX() == 6 && oCell.GetY() == 0) {
          bFound = true;
          break;
        }
      }
      if (!bFound) {
        fail("Didn't find cell 6,0.");
      }

      //14
      bFound = false;
      for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
        oCell = oHarvestData.GetCell(i);
        if (oCell.GetX() == 7 && oCell.GetY() == 1) {
          bFound = true;
          break;
        }
      }
      if (!bFound) {
        fail("Didn't find cell 7,1.");
      }

      bFound = false;
      for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
        oCell = oHarvestData.GetCell(i);
        if (oCell.GetX() == 7 && oCell.GetY() == 2) {
          bFound = true;
          break;
        }
      }
      if (!bFound) {
        fail("Didn't find cell 7,2.");
      }

      //15
      bFound = false;
      for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
        oCell = oHarvestData.GetCell(i);
        if (oCell.GetX() == 6 && oCell.GetY() == 1) {
          bFound = true;
          break;
        }
      }
      if (!bFound) {
        fail("Didn't find cell 6,1.");
      }

      bFound = false;
      for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
        oCell = oHarvestData.GetCell(i);
        if (oCell.GetX() == 6 && oCell.GetY() == 2) {
          bFound = true;
          break;
        }
      }
      if (!bFound) {
        fail("Didn't find cell 6,2.");
      }

      //16
      bFound = false;
      for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
        oCell = oHarvestData.GetCell(i);
        if (oCell.GetX() == 7 && oCell.GetY() == 0) {
          bFound = true;
          break;
        }
      }
      if (!bFound) {
        fail("Didn't find cell 7,0.");
      }

      //17
      bFound = false;
      for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
        oCell = oHarvestData.GetCell(i);
        if (oCell.GetX() == 8 && oCell.GetY() == 1) {
          bFound = true;
          break;
        }
      }
      if (!bFound) {
        fail("Didn't find cell 8,1.");
      }

      bFound = false;
      for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
        oCell = oHarvestData.GetCell(i);
        if (oCell.GetX() == 8 && oCell.GetY() == 2) {
          bFound = true;
          break;
        }
      }
      if (!bFound) {
        fail("Didn't find cell 8,2.");
      }

      //18
      bFound = false;
      for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
        oCell = oHarvestData.GetCell(i);
        if (oCell.GetX() == 8 && oCell.GetY() == 0) {
          bFound = true;
          break;
        }
      }
      if (!bFound) {
        fail("Didn't find cell 8,0.");
      }

      bFound = false;
      for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
        oCell = oHarvestData.GetCell(i);
        if (oCell.GetX() == 9 && oCell.GetY() == 0) {
          bFound = true;
          break;
        }
      }
      if (!bFound) {
        fail("Didn't find cell 9,0.");
      }

      bFound = false;
      for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
        oCell = oHarvestData.GetCell(i);
        if (oCell.GetX() == 9 && oCell.GetY() == 1) {
          bFound = true;
          break;
        }
      }
      if (!bFound) {
        fail("Didn't find cell 9,1.");
      }

      bFound = false;
      for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
        oCell = oHarvestData.GetCell(i);
        if (oCell.GetX() == 9 && oCell.GetY() == 2) {
          bFound = true;
          break;
        }
      }
      if (!bFound) {
        fail("Didn't find cell 9,2.");
      }

      bFound = false;
      for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
        oCell = oHarvestData.GetCell(i);
        if (oCell.GetX() == 10 && oCell.GetY() == 0) {
          bFound = true;
          break;
        }
      }
      if (!bFound) {
        fail("Didn't find cell 10,0.");
      }

      bFound = false;
      for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
        oCell = oHarvestData.GetCell(i);
        if (oCell.GetX() == 10 && oCell.GetY() == 1) {
          bFound = true;
          break;
        }
      }
      if (!bFound) {
        fail("Didn't find cell 10,1.");
      }

      bFound = false;
      for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
        oCell = oHarvestData.GetCell(i);
        if (oCell.GetX() == 10 && oCell.GetY() == 2) {
          bFound = true;
          break;
        }
      }
      if (!bFound) {
        fail("Didn't find cell 10,2.");
      }

      bFound = false;
      for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
        oCell = oHarvestData.GetCell(i);
        if (oCell.GetX() == 11 && oCell.GetY() == 0) {
          bFound = true;
          break;
        }
      }
      if (!bFound) {
        fail("Didn't find cell 11,0.");
      }

      bFound = false;
      for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
        oCell = oHarvestData.GetCell(i);
        if (oCell.GetX() == 11 && oCell.GetY() == 1) {
          bFound = true;
          break;
        }
      }
      if (!bFound) {
        fail("Didn't find cell 11,1.");
      }

      bFound = false;
      for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
        oCell = oHarvestData.GetCell(i);
        if (oCell.GetX() == 11 && oCell.GetY() == 2) {
          bFound = true;
          break;
        }
      }
      if (!bFound) {
        fail("Didn't find cell 11,2.");
      }

      bFound = false;
      for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
        oCell = oHarvestData.GetCell(i);
        if (oCell.GetX() == 12 && oCell.GetY() == 0) {
          bFound = true;
          break;
        }
      }
      if (!bFound) {
        fail("Didn't find cell 12,0.");
      }

      bFound = false;
      for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
        oCell = oHarvestData.GetCell(i);
        if (oCell.GetX() == 12 && oCell.GetY() == 1) {
          bFound = true;
          break;
        }
      }
      if (!bFound) {
        fail("Didn't find cell 12,1.");
      }

      bFound = false;
      for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
        oCell = oHarvestData.GetCell(i);
        if (oCell.GetX() == 12 && oCell.GetY() == 2) {
          bFound = true;
          break;
        }
      }
      if (!bFound) {
        fail("Didn't find cell 12,2.");
      }

      bFound = false;
      for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
        oCell = oHarvestData.GetCell(i);
        if (oCell.GetX() == 13 && oCell.GetY() == 0) {
          bFound = true;
          break;
        }
      }
      if (!bFound) {
        fail("Didn't find cell 13,0.");
      }

      bFound = false;
      for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
        oCell = oHarvestData.GetCell(i);
        if (oCell.GetX() == 13 && oCell.GetY() == 1) {
          bFound = true;
          break;
        }
      }
      if (!bFound) {
        fail("Didn't find cell 13,1.");
      }

      bFound = false;
      for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
        oCell = oHarvestData.GetCell(i);
        if (oCell.GetX() == 13 && oCell.GetY() == 2) {
          bFound = true;
          break;
        }
      }
      if (!bFound) {
        fail("Didn't find cell 13,2.");
      }

      bFound = false;
      for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
        oCell = oHarvestData.GetCell(i);
        if (oCell.GetX() == 14 && oCell.GetY() == 0) {
          bFound = true;
          break;
        }
      }
      if (!bFound) {
        fail("Didn't find cell 14,0.");
      }

      bFound = false;
      for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
        oCell = oHarvestData.GetCell(i);
        if (oCell.GetX() == 14 && oCell.GetY() == 1) {
          bFound = true;
          break;
        }
      }
      if (!bFound) {
        fail("Didn't find cell 4,1.");
      }

      bFound = false;
      for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
        oCell = oHarvestData.GetCell(i);
        if (oCell.GetX() == 14 && oCell.GetY() == 2) {
          bFound = true;
          break;
        }
      }
      if (!bFound) {
        fail("Didn't find cell 14,2.");
      }


      //19
      bFound = false;
      for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
        oCell = oHarvestData.GetCell(i);
        if (oCell.GetX() == 15 && oCell.GetY() == 2) {
          bFound = true;
          break;
        }
      }
      if (!bFound) {
        fail("Didn't find cell 15,2.");
      }

      bFound = false;
      for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
        oCell = oHarvestData.GetCell(i);
        if (oCell.GetX() == 16 && oCell.GetY() == 0) {
          bFound = true;
          break;
        }
      }
      if (!bFound) {
        fail("Didn't find cell 16,0.");
      }

      bFound = false;
      for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
        oCell = oHarvestData.GetCell(i);
        if (oCell.GetX() == 16 && oCell.GetY() == 1) {
          bFound = true;
          break;
        }
      }
      if (!bFound) {
        fail("Didn't find cell 16,1.");
      }

      bFound = false;
      for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
        oCell = oHarvestData.GetCell(i);
        if (oCell.GetX() == 16 && oCell.GetY() == 2) {
          bFound = true;
          break;
        }
      }
      if (!bFound) {
        fail("Didn't find cell 16,2.");
      }

      bFound = false;
      for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
        oCell = oHarvestData.GetCell(i);
        if (oCell.GetX() == 17 && oCell.GetY() == 0) {
          bFound = true;
          break;
        }
      }
      if (!bFound) {
        fail("Didn't find cell 17,0.");
      }

      bFound = false;
      for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
        oCell = oHarvestData.GetCell(i);
        if (oCell.GetX() == 17 && oCell.GetY() == 1) {
          bFound = true;
          break;
        }
      }
      if (!bFound) {
        fail("Didn't find cell 17,1.");
      }

      bFound = false;
      for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
        oCell = oHarvestData.GetCell(i);
        if (oCell.GetX() == 17 && oCell.GetY() == 2) {
          bFound = true;
          break;
        }
      }
      if (!bFound) {
        fail("Didn't find cell 17,2.");
      }

      bFound = false;
      for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
        oCell = oHarvestData.GetCell(i);
        if (oCell.GetX() == 18 && oCell.GetY() == 0) {
          bFound = true;
          break;
        }
      }
      if (!bFound) {
        fail("Didn't find cell 18,0.");
      }

      bFound = false;
      for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
        oCell = oHarvestData.GetCell(i);
        if (oCell.GetX() == 18 && oCell.GetY() == 1) {
          bFound = true;
          break;
        }
      }
      if (!bFound) {
        fail("Didn't find cell 18,1.");
      }

      bFound = false;
      for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
        oCell = oHarvestData.GetCell(i);
        if (oCell.GetX() == 18 && oCell.GetY() == 2) {
          bFound = true;
          break;
        }
      }
      if (!bFound) {
        fail("Didn't find cell 18,2.");
      }

      bFound = false;
      for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
        oCell = oHarvestData.GetCell(i);
        if (oCell.GetX() == 19 && oCell.GetY() == 0) {
          bFound = true;
          break;
        }
      }
      if (!bFound) {
        fail("Didn't find cell 19,0.");
      }

      bFound = false;
      for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
        oCell = oHarvestData.GetCell(i);
        if (oCell.GetX() == 19 && oCell.GetY() == 1) {
          bFound = true;
          break;
        }
      }
      if (!bFound) {
        fail("Didn't find cell 19,1.");
      }

      bFound = false;
      for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
        oCell = oHarvestData.GetCell(i);
        if (oCell.GetX() == 19 && oCell.GetY() == 2) {
          bFound = true;
          break;
        }
      }
      if (!bFound) {
        fail("Didn't find cell 19,2.");
      }

      bFound = false;
      for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
        oCell = oHarvestData.GetCell(i);
        if (oCell.GetX() == 32 && oCell.GetY() == 4) {
          bFound = true;
          break;
        }
      }
      if (!bFound) {
        fail("Didn't find cell 32,4.");
      }

      bFound = false;
      for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
        oCell = oHarvestData.GetCell(i);
        if (oCell.GetX() == 32 && oCell.GetY() == 5) {
          bFound = true;
          break;
        }
      }
      if (!bFound) {
        fail("Didn't find cell 32,5.");
      }

      bFound = false;
      for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
        oCell = oHarvestData.GetCell(i);
        if (oCell.GetX() == 32 && oCell.GetY() == 6) {
          bFound = true;
          break;
        }
      }
      if (!bFound) {
        fail("Didn't find cell 32,6.");
      }

      bFound = false;
      for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
        oCell = oHarvestData.GetCell(i);
        if (oCell.GetX() == 33 && oCell.GetY() == 4) {
          bFound = true;
          break;
        }
      }
      if (!bFound) {
        fail("Didn't find cell 33,4.");
      }

      bFound = false;
      for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
        oCell = oHarvestData.GetCell(i);
        if (oCell.GetX() == 33 && oCell.GetY() == 5) {
          bFound = true;
          break;
        }
      }
      if (!bFound) {
        fail("Didn't find cell 33,5.");
      }

      bFound = false;
      for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
        oCell = oHarvestData.GetCell(i);
        if (oCell.GetX() == 33 && oCell.GetY() == 6) {
          bFound = true;
          break;
        }
      }
      if (!bFound) {
        fail("Didn't find cell 33,6.");
      }

      bFound = false;
      for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
        oCell = oHarvestData.GetCell(i);
        if (oCell.GetX() == 34 && oCell.GetY() == 4) {
          bFound = true;
          break;
        }
      }
      if (!bFound) {
        fail("Didn't find cell 34,4.");
      }

      bFound = false;
      for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
        oCell = oHarvestData.GetCell(i);
        if (oCell.GetX() == 34 && oCell.GetY() == 5) {
          bFound = true;
          break;
        }
      }
      if (!bFound) {
        fail("Didn't find cell 34,5.");
      }

      bFound = false;
      for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
        oCell = oHarvestData.GetCell(i);
        if (oCell.GetX() == 34 && oCell.GetY() == 6) {
          bFound = true;
          break;
        }
      }
      if (!bFound) {
        fail("Didn't find cell 34,6.");
      }

      bFound = false;
      for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
        oCell = oHarvestData.GetCell(i);
        if (oCell.GetX() == 35 && oCell.GetY() == 4) {
          bFound = true;
          break;
        }
      }
      if (!bFound) {
        fail("Didn't find cell 35,4.");
      }

      bFound = false;
      for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
        oCell = oHarvestData.GetCell(i);
        if (oCell.GetX() == 35 && oCell.GetY() == 5) {
          bFound = true;
          break;
        }
      }
      if (!bFound) {
        fail("Didn't find cell 35,5.");
      }

      bFound = false;
      for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
        oCell = oHarvestData.GetCell(i);
        if (oCell.GetX() == 35 && oCell.GetY() == 6) {
          bFound = true;
          break;
        }
      }
      if (!bFound) {
        fail("Didn't find cell 35,6.");
      }

      bFound = false;
      for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
        oCell = oHarvestData.GetCell(i);
        if (oCell.GetX() == 58 && oCell.GetY() == 4) {
          bFound = true;
          break;
        }
      }
      if (!bFound) {
        fail("Didn't find cell 58,4.");
      }

      bFound = false;
      for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
        oCell = oHarvestData.GetCell(i);
        if (oCell.GetX() == 58 && oCell.GetY() == 5) {
          bFound = true;
          break;
        }
      }
      if (!bFound) {
        fail("Didn't find cell 58,5.");
      }

      bFound = false;
      for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
        oCell = oHarvestData.GetCell(i);
        if (oCell.GetX() == 58 && oCell.GetY() == 6) {
          bFound = true;
          break;
        }
      }
      if (!bFound) {
        fail("Didn't find cell 58,6.");
      }

      bFound = false;
      for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
        oCell = oHarvestData.GetCell(i);
        if (oCell.GetX() == 59 && oCell.GetY() == 4) {
          bFound = true;
          break;
        }
      }
      if (!bFound) {
        fail("Didn't find cell 59,4.");
      }

      bFound = false;
      for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
        oCell = oHarvestData.GetCell(i);
        if (oCell.GetX() == 59 && oCell.GetY() == 5) {
          bFound = true;
          break;
        }
      }
      if (!bFound) {
        fail("Didn't find cell 59,5.");
      }

      bFound = false;
      for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
        oCell = oHarvestData.GetCell(i);
        if (oCell.GetX() == 59 && oCell.GetY() == 6) {
          bFound = true;
          break;
        }
      }
      if (!bFound) {
        fail("Didn't find cell 59,6.");
      }

      bFound = false;
      for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
        oCell = oHarvestData.GetCell(i);
        if (oCell.GetX() == 60 && oCell.GetY() == 4) {
          bFound = true;
          break;
        }
      }
      if (!bFound) {
        fail("Didn't find cell 60,4.");
      }

      bFound = false;
      for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
        oCell = oHarvestData.GetCell(i);
        if (oCell.GetX() == 60 && oCell.GetY() == 5) {
          bFound = true;
          break;
        }
      }
      if (!bFound) {
        fail("Didn't find cell 60,5.");
      }

      bFound = false;
      for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
        oCell = oHarvestData.GetCell(i);
        if (oCell.GetX() == 60 && oCell.GetY() == 6) {
          bFound = true;
          break;
        }
      }
      if (!bFound) {
        fail("Didn't find cell 60,6.");
      }

      bFound = false;
      for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
        oCell = oHarvestData.GetCell(i);
        if (oCell.GetX() == 61 && oCell.GetY() == 4) {
          bFound = true;
          break;
        }
      }
      if (!bFound) {
        fail("Didn't find cell 61,4.");
      }

      bFound = false;
      for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
        oCell = oHarvestData.GetCell(i);
        if (oCell.GetX() == 61 && oCell.GetY() == 5) {
          bFound = true;
          break;
        }
      }
      if (!bFound) {
        fail("Didn't find cell 61,5.");
      }

      bFound = false;
      for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
        oCell = oHarvestData.GetCell(i);
        if (oCell.GetX() == 61 && oCell.GetY() == 6) {
          bFound = true;
          break;
        }
      }
      if (!bFound) {
        fail("Didn't find cell 61,6.");
      }

      bFound = false;
      for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
        oCell = oHarvestData.GetCell(i);
        if (oCell.GetX() == 13 && oCell.GetY() == 6) {
          bFound = true;
          break;
        }
      }
      if (!bFound) {
        fail("Didn't find cell 13,6.");
      }

      bFound = false;
      for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
        oCell = oHarvestData.GetCell(i);
        if (oCell.GetX() == 13 && oCell.GetY() == 7) {
          bFound = true;
          break;
        }
      }
      if (!bFound) {
        fail("Didn't find cell 13,7.");
      }

      bFound = false;
      for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
        oCell = oHarvestData.GetCell(i);
        if (oCell.GetX() == 13 && oCell.GetY() == 8) {
          bFound = true;
          break;
        }
      }
      if (!bFound) {
        fail("Didn't find cell 13,8.");
      }

      bFound = false;
      for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
        oCell = oHarvestData.GetCell(i);
        if (oCell.GetX() == 14 && oCell.GetY() == 6) {
          bFound = true;
          break;
        }
      }
      if (!bFound) {
        fail("Didn't find cell 14,6.");
      }

      bFound = false;
      for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
        oCell = oHarvestData.GetCell(i);
        if (oCell.GetX() == 14 && oCell.GetY() == 7) {
          bFound = true;
          break;
        }
      }
      if (!bFound) {
        fail("Didn't find cell 14,7.");
      }

      bFound = false;
      for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
        oCell = oHarvestData.GetCell(i);
        if (oCell.GetX() == 14 && oCell.GetY() == 8) {
          bFound = true;
          break;
        }
      }
      if (!bFound) {
        fail("Didn't find cell 14,8.");
      }

      bFound = false;
      for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
        oCell = oHarvestData.GetCell(i);
        if (oCell.GetX() == 15 && oCell.GetY() == 6) {
          bFound = true;
          break;
        }
      }
      if (!bFound) {
        fail("Didn't find cell 15,6.");
      }

      bFound = false;
      for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
        oCell = oHarvestData.GetCell(i);
        if (oCell.GetX() == 15 && oCell.GetY() == 7) {
          bFound = true;
          break;
        }
      }
      if (!bFound) {
        fail("Didn't find cell 15,7.");
      }

      bFound = false;
      for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
        oCell = oHarvestData.GetCell(i);
        if (oCell.GetX() == 15 && oCell.GetY() == 8) {
          bFound = true;
          break;
        }
      }
      if (!bFound) {
        fail("Didn't find cell 15,8.");
      }

      bFound = false;
      for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
        oCell = oHarvestData.GetCell(i);
        if (oCell.GetX() == 16 && oCell.GetY() == 6) {
          bFound = true;
          break;
        }
      }
      if (!bFound) {
        fail("Didn't find cell 16,6.");
      }

      bFound = false;
      for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
        oCell = oHarvestData.GetCell(i);
        if (oCell.GetX() == 16 && oCell.GetY() == 7) {
          bFound = true;
          break;
        }
      }
      if (!bFound) {
        fail("Didn't find cell 16,7.");
      }

      bFound = false;
      for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
        oCell = oHarvestData.GetCell(i);
        if (oCell.GetX() == 16 && oCell.GetY() == 8) {
          bFound = true;
          break;
        }
      }
      if (!bFound) {
        fail("Didn't find cell 16,8.");
      }

      bFound = false;
      for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
        oCell = oHarvestData.GetCell(i);
        if (oCell.GetX() == 64 && oCell.GetY() == 44) {
          bFound = true;
          break;
        }
      }
      if (!bFound) {
        fail("Didn't find cell 64,44.");
      }

      bFound = false;
      for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
        oCell = oHarvestData.GetCell(i);
        if (oCell.GetX() == 61 && oCell.GetY() == 42) {
          bFound = true;
          break;
        }
      }
      if (!bFound) {
        fail("Didn't find cell 61,42.");
      }

      bFound = false;
      for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
        oCell = oHarvestData.GetCell(i);
        if (oCell.GetX() == 62 && oCell.GetY() == 42) {
          bFound = true;
          break;
        }
      }
      if (!bFound) {
        fail("Didn't find cell 62,42.");
      }

      bFound = false;
      for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
        oCell = oHarvestData.GetCell(i);
        if (oCell.GetX() == 63 && oCell.GetY() == 42) {
          bFound = true;
          break;
        }
      }
      if (!bFound) {
        fail("Didn't find cell 63,42.");
      }

      bFound = false;
      for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
        oCell = oHarvestData.GetCell(i);
        if (oCell.GetX() == 64 && oCell.GetY() == 42) {
          bFound = true;
          break;
        }
      }
      if (!bFound) {
        fail("Didn't find cell 64,42.");
      }

      bFound = false;
      for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
        oCell = oHarvestData.GetCell(i);
        if (oCell.GetX() == 61 && oCell.GetY() == 43) {
          bFound = true;
          break;
        }
      }
      if (!bFound) {
        fail("Didn't find cell 61,43.");
      }

      bFound = false;
      for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
        oCell = oHarvestData.GetCell(i);
        if (oCell.GetX() == 62 && oCell.GetY() == 43) {
          bFound = true;
          break;
        }
      }
      if (!bFound) {
        fail("Didn't find cell 62,43.");
      }

      bFound = false;
      for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
        oCell = oHarvestData.GetCell(i);
        if (oCell.GetX() == 63 && oCell.GetY() == 43) {
          bFound = true;
          break;
        }
      }
      if (!bFound) {
        fail("Didn't find cell 63,43.");
      }

      bFound = false;
      for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
        oCell = oHarvestData.GetCell(i);
        if (oCell.GetX() == 64 && oCell.GetY() == 43) {
          bFound = true;
          break;
        }
      }
      if (!bFound) {
        fail("Didn't find cell 64,43.");
      }

      bFound = false;
      for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
        oCell = oHarvestData.GetCell(i);
        if (oCell.GetX() == 61 && oCell.GetY() == 44) {
          bFound = true;
          break;
        }
      }
      if (!bFound) {
        fail("Didn't find cell 61,44.");
      }

      bFound = false;
      for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
        oCell = oHarvestData.GetCell(i);
        if (oCell.GetX() == 62 && oCell.GetY() == 44) {
          bFound = true;
          break;
        }
      }
      if (!bFound) {
        fail("Didn't find cell 62,44.");
      }

      bFound = false;
      for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
        oCell = oHarvestData.GetCell(i);
        if (oCell.GetX() == 63 && oCell.GetY() == 44) {
          bFound = true;
          break;
        }
      }
      if (!bFound) {
        fail("Didn't find cell 63,44.");
      }
    }
    catch (ModelException oErr) {
      fail("Normal processing for TransformOldGridCell failed with message " +
           oErr.sMessage);
    }
  }

  public void testTransformOldGridCell5() {
    try {
      float fOldXGridLength = 8;
      float fOldYGridLength = 10;
      int iNumOldXGrids = 20;
      int iNumOldYGrids = 20;
      Plot oPlot = new Plot(null);
      HarvestData oHarvestData = new HarvestData();
      oPlot.SetPlotXLength(200);
      oPlot.SetPlotYLength(160);
      int[] p_iOldCells = {
         0, 1, 2, 3, 4, 5, 50, 58, 64, 399};
     int i, j;
     boolean bFound;


      Cell[] p_oActualReturn;
      Cell oCell;

      //---------------------------------------------------------------------
      //Case:  X cell length equal to 8 and Y cell length greater than 8
      //---------------------------------------------------------------------

     //Transform and add a list of old cells
     for (i = 0; i < p_iOldCells.length; i++) {
       p_oActualReturn = Cell.TransformOldGridCell(p_iOldCells[i],
           fOldXGridLength,
           fOldYGridLength,
           iNumOldXGrids, iNumOldYGrids,
           oPlot);
       for (j = 0; j < p_oActualReturn.length; j++) {
         oHarvestData.AddCell(p_oActualReturn[j]);
       }
     }

     assertEquals(16, oHarvestData.GetNumberOfCells());

     //Make sure all of the expected cells are on the list
     bFound = false;
     for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
       oCell = oHarvestData.GetCell(i);
       if (oCell.GetX() == 0 && oCell.GetY() == 0) {
         bFound = true;
         break;
       }
     }
     if (!bFound) {
       fail("Didn't find cell 0,0.");
     }

     bFound = false;
     for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
       oCell = oHarvestData.GetCell(i);
       if (oCell.GetX() == 1 && oCell.GetY() == 0) {
         bFound = true;
         break;
       }
     }
     if (!bFound) {
       fail("Didn't find cell 1,0.");
     }

     bFound = false;
     for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
       oCell = oHarvestData.GetCell(i);
       if (oCell.GetX() == 2 && oCell.GetY() == 0) {
         bFound = true;
         break;
       }
     }
     if (!bFound) {
       fail("Didn't find cell 2,0.");
     }

     bFound = false;
     for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
       oCell = oHarvestData.GetCell(i);
       if (oCell.GetX() == 3 && oCell.GetY() == 0) {
         bFound = true;
         break;
       }
     }
     if (!bFound) {
       fail("Didn't find cell 3,0.");
     }

     bFound = false;
     for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
       oCell = oHarvestData.GetCell(i);
       if (oCell.GetX() == 4 && oCell.GetY() == 0) {
         bFound = true;
         break;
       }
     }
     if (!bFound) {
       fail("Didn't find cell 4,0.");
     }

     bFound = false;
     for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
       oCell = oHarvestData.GetCell(i);
       if (oCell.GetX() == 5 && oCell.GetY() == 0) {
         bFound = true;
         break;
       }
     }
     if (!bFound) {
       fail("Didn't find cell 5,0.");
     }

     bFound = false;
     for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
       oCell = oHarvestData.GetCell(i);
       if (oCell.GetX() == 6 && oCell.GetY() == 0) {
         bFound = true;
         break;
       }
     }
     if (!bFound) {
       fail("Didn't find cell 6,0.");
     }

     bFound = false;
     for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
       oCell = oHarvestData.GetCell(i);
       if (oCell.GetX() == 7 && oCell.GetY() == 0) {
         bFound = true;
         break;
       }
     }
     if (!bFound) {
       fail("Didn't find cell 7,0.");
     }

     bFound = false;
     for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
       oCell = oHarvestData.GetCell(i);
       if (oCell.GetX() == 12 && oCell.GetY() == 2) {
         bFound = true;
         break;
       }
     }
     if (!bFound) {
       fail("Didn't find cell 12,2.");
     }

     bFound = false;
     for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
       oCell = oHarvestData.GetCell(i);
       if (oCell.GetX() == 13 && oCell.GetY() == 2) {
         bFound = true;
         break;
       }
     }
     if (!bFound) {
       fail("Didn't find cell 13,2.");
     }

     bFound = false;
     for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
       oCell = oHarvestData.GetCell(i);
       if (oCell.GetX() == 22 && oCell.GetY() == 2) {
         bFound = true;
         break;
       }
     }
     if (!bFound) {
       fail("Didn't find cell 22,2.");
     }

     bFound = false;
     for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
       oCell = oHarvestData.GetCell(i);
       if (oCell.GetX() == 23 && oCell.GetY() == 2) {
         bFound = true;
         break;
       }
     }
     if (!bFound) {
       fail("Didn't find cell 23,2.");
     }

     bFound = false;
     for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
       oCell = oHarvestData.GetCell(i);
       if (oCell.GetX() == 5 && oCell.GetY() == 3) {
         bFound = true;
         break;
       }
     }
     if (!bFound) {
       fail("Didn't find cell 5,3.");
     }

     bFound = false;
     for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
       oCell = oHarvestData.GetCell(i);
       if (oCell.GetX() == 6 && oCell.GetY() == 3) {
         bFound = true;
         break;
       }
     }
     if (!bFound) {
       fail("Didn't find cell 6,3.");
     }

     bFound = false;
     for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
       oCell = oHarvestData.GetCell(i);
       if (oCell.GetX() == 24 && oCell.GetY() == 19) {
         bFound = true;
         break;
       }
     }
     if (!bFound) {
       fail("Didn't find cell 24,19.");
     }

     bFound = false;
     for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
       oCell = oHarvestData.GetCell(i);
       if (oCell.GetX() == 23 && oCell.GetY() == 19) {
         bFound = true;
         break;
       }
     }
     if (!bFound) {
       fail("Didn't find cell 23,19.");
     }
    }
    catch (ModelException oErr) {
      fail("Normal processing for TransformOldGridCell failed with message " +
           oErr.sMessage);
    }
  }

  public void testTransformOldGridCell6() {
    try {
      float fOldXGridLength = 10;
      float fOldYGridLength = 8;
      int iNumOldXGrids = 20;
      int iNumOldYGrids = 20;
      Plot oPlot = new Plot(null);
      HarvestData oHarvestData = new HarvestData();
      oPlot.SetPlotXLength(160);
      oPlot.SetPlotYLength(200);
      int[] p_iOldCells = {
         0, 1, 2, 3, 4, 5, 50, 58, 64, 399};
     int i, j;
     boolean bFound;


      Cell[] p_oActualReturn;
      Cell oCell;

      //---------------------------------------------------------------------
      //Case:  Y cell length equal to 8 and X cell length greater than 8
      //---------------------------------------------------------------------

     //Transform and add a list of old cells
     for (i = 0; i < p_iOldCells.length; i++) {
       p_oActualReturn = Cell.TransformOldGridCell(p_iOldCells[i],
           fOldXGridLength,
           fOldYGridLength,
           iNumOldXGrids, iNumOldYGrids,
           oPlot);
       for (j = 0; j < p_oActualReturn.length; j++) {
         oHarvestData.AddCell(p_oActualReturn[j]);
       }
     }

     assertEquals(20, oHarvestData.GetNumberOfCells());

     //Make sure all of the expected cells are on the list
     bFound = false;
     for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
       oCell = oHarvestData.GetCell(i);
       if (oCell.GetX() == 0 && oCell.GetY() == 0) {
         bFound = true;
         break;
       }
     }
     if (!bFound) {
       fail("Didn't find cell 0,0.");
     }

     bFound = false;
     for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
       oCell = oHarvestData.GetCell(i);
       if (oCell.GetX() == 0 && oCell.GetY() == 1) {
         bFound = true;
         break;
       }
     }
     if (!bFound) {
       fail("Didn't find cell 0,1.");
     }

     bFound = false;
     for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
       oCell = oHarvestData.GetCell(i);
       if (oCell.GetX() == 1 && oCell.GetY() == 0) {
         bFound = true;
         break;
       }
     }
     if (!bFound) {
       fail("Didn't find cell 1,0.");
     }

     bFound = false;
     for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
       oCell = oHarvestData.GetCell(i);
       if (oCell.GetX() == 1 && oCell.GetY() == 1) {
         bFound = true;
         break;
       }
     }
     if (!bFound) {
       fail("Didn't find cell 1,1.");
     }


     bFound = false;
     for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
       oCell = oHarvestData.GetCell(i);
       if (oCell.GetX() == 2 && oCell.GetY() == 0) {
         bFound = true;
         break;
       }
     }
     if (!bFound) {
       fail("Didn't find cell 2,0.");
     }

     bFound = false;
     for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
       oCell = oHarvestData.GetCell(i);
       if (oCell.GetX() == 2 && oCell.GetY() == 1) {
         bFound = true;
         break;
       }
     }
     if (!bFound) {
       fail("Didn't find cell 2,1.");
     }

     bFound = false;
     for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
       oCell = oHarvestData.GetCell(i);
       if (oCell.GetX() == 3 && oCell.GetY() == 0) {
         bFound = true;
         break;
       }
     }
     if (!bFound) {
       fail("Didn't find cell 3,0.");
     }

     bFound = false;
     for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
       oCell = oHarvestData.GetCell(i);
       if (oCell.GetX() == 3 && oCell.GetY() == 1) {
         bFound = true;
         break;
       }
     }
     if (!bFound) {
       fail("Didn't find cell 3,1.");
     }

     bFound = false;
     for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
       oCell = oHarvestData.GetCell(i);
       if (oCell.GetX() == 4 && oCell.GetY() == 0) {
         bFound = true;
         break;
       }
     }
     if (!bFound) {
       fail("Didn't find cell 4,0.");
     }

     bFound = false;
     for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
       oCell = oHarvestData.GetCell(i);
       if (oCell.GetX() == 4 && oCell.GetY() == 1) {
         bFound = true;
         break;
       }
     }
     if (!bFound) {
       fail("Didn't find cell 4,1.");
     }

     bFound = false;
     for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
       oCell = oHarvestData.GetCell(i);
       if (oCell.GetX() == 5 && oCell.GetY() == 0) {
         bFound = true;
         break;
       }
     }
     if (!bFound) {
       fail("Didn't find cell 5,0.");
     }

     bFound = false;
     for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
       oCell = oHarvestData.GetCell(i);
       if (oCell.GetX() == 5 && oCell.GetY() == 1) {
         bFound = true;
         break;
       }
     }
     if (!bFound) {
       fail("Didn't find cell 5,1.");
     }

     bFound = false;
     for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
       oCell = oHarvestData.GetCell(i);
       if (oCell.GetX() == 10 && oCell.GetY() == 2) {
         bFound = true;
         break;
       }
     }
     if (!bFound) {
       fail("Didn't find cell 10,2.");
     }

     bFound = false;
     for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
       oCell = oHarvestData.GetCell(i);
       if (oCell.GetX() == 10 && oCell.GetY() == 3) {
         bFound = true;
         break;
       }
     }
     if (!bFound) {
       fail("Didn't find cell 10,3.");
     }

     bFound = false;
     for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
       oCell = oHarvestData.GetCell(i);
       if (oCell.GetX() == 18 && oCell.GetY() == 2) {
         bFound = true;
         break;
       }
     }
     if (!bFound) {
       fail("Didn't find cell 18,2.");
     }

     bFound = false;
     for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
       oCell = oHarvestData.GetCell(i);
       if (oCell.GetX() == 18 && oCell.GetY() == 3) {
         bFound = true;
         break;
       }
     }
     if (!bFound) {
       fail("Didn't find cell 18,3.");
     }

     bFound = false;
     for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
       oCell = oHarvestData.GetCell(i);
       if (oCell.GetX() == 4 && oCell.GetY() == 3) {
         bFound = true;
         break;
       }
     }
     if (!bFound) {
       fail("Didn't find cell 4,3.");
     }

     bFound = false;
     for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
       oCell = oHarvestData.GetCell(i);
       if (oCell.GetX() == 4 && oCell.GetY() == 4) {
         bFound = true;
         break;
       }
     }
     if (!bFound) {
       fail("Didn't find cell 4,4.");
     }

     bFound = false;
     for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
       oCell = oHarvestData.GetCell(i);
       if (oCell.GetX() == 19 && oCell.GetY() == 24) {
         bFound = true;
         break;
       }
     }
     if (!bFound) {
       fail("Didn't find cell 19,24.");
     }

     bFound = false;
     for (i = 0; i < oHarvestData.GetNumberOfCells(); i++) {
       oCell = oHarvestData.GetCell(i);
       if (oCell.GetX() == 19 && oCell.GetY() == 23) {
         bFound = true;
         break;
       }
     }
     if (!bFound) {
       fail("Didn't find cell 19,23.");
     }
    }
    catch (ModelException oErr) {
      fail("Normal processing for TransformOldGridCell failed with message " +
           oErr.sMessage);
    }
  }


}
