package javawrapper;

import junit.framework.*;

/**
 * Tests the HarvestData class.
 * <p>Copyright: Copyright (c) Charles D. Canham 2003</p>
 * <p>Company: Institute of Ecosystem Studies</p>
 * @author Lora E. Murphy
 * @version 1.0
 *
 * <br>Edit history:
 * <br>------------------
 * <br>April 28, 2004: Submitted in beta version (LEM)
 */
public class TestHarvestData
    extends TestCase {
  private HarvestData m_oHarvestData = null;

  protected void setUp() throws Exception {
    super.setUp();
    m_oHarvestData = new HarvestData();
  }

  protected void tearDown() throws Exception {
    m_oHarvestData = null;
    super.tearDown();
  }

  public void testGetSetCutAmountType() {
    int iCut,
        iActual;

    try {
      //No value has been set
      iCut = -1;
      iActual = m_oHarvestData.GetCutType();
      assertEquals(iActual, iCut);

      iCut = DisturbanceBehaviors.ABSOLUTE_BASAL_AREA;
      m_oHarvestData.SetCutAmountType(iCut);
      iActual = m_oHarvestData.GetCutAmountType();
      assertEquals(iActual, iCut);

      iCut = DisturbanceBehaviors.ABSOLUTE_DENSITY;
      m_oHarvestData.SetCutAmountType(iCut);
      iActual = m_oHarvestData.GetCutAmountType();
      assertEquals(iActual, iCut);

      iCut = DisturbanceBehaviors.PERCENTAGE_BASAL_AREA;
      m_oHarvestData.SetCutAmountType(iCut);
      iActual = m_oHarvestData.GetCutAmountType();
      assertEquals(iActual, iCut);

      iCut = DisturbanceBehaviors.PERCENTAGE_DENSITY;
      m_oHarvestData.SetCutAmountType(iCut);
      iActual = m_oHarvestData.GetCutAmountType();
      assertEquals(iActual, iCut);

      System.out.println(
          "Normal processing succeeded for SetCutAmountType and GetCutAmountType.");
    }
    catch (ModelException oErr) {
      fail("Normal processing for SetCutAmountType failed with message " +
           oErr.sMessage);
    }

    try {
      iCut = 17;
      m_oHarvestData.SetCutAmountType(iCut);
      fail("SetCutAmountType did not catch a cut type value of " +
           String.valueOf(iCut));
    }
    catch (ModelException oErr) {
      System.out.println("Exception processing succeeded for SetCutAmountType.");
    }
  }

  public void testGetSetCutType() {
    int iCut,
        iActual;

    try {
      //No value has been set
      iCut = -1;
      iActual = m_oHarvestData.GetCutType();
      assertEquals(iActual, iCut);

      iCut = DisturbanceBehaviors.PARTIAL_CUT;
      m_oHarvestData.SetCutType(iCut);
      iActual = m_oHarvestData.GetCutType();
      assertEquals(iActual, iCut);

      iCut = DisturbanceBehaviors.GAP_CUT;
      m_oHarvestData.SetCutType(iCut);
      iActual = m_oHarvestData.GetCutType();
      assertEquals(iActual, iCut);

      iCut = DisturbanceBehaviors.CLEAR_CUT;
      m_oHarvestData.SetCutType(iCut);
      iActual = m_oHarvestData.GetCutType();
      assertEquals(iActual, iCut);

      System.out.println(
          "Normal processing succeeded for SetCutType and GetCutType.");
    }
    catch (ModelException oErr) {
      fail("Normal processing for SetCutType failed with message " +
           oErr.sMessage);
    }

    try {
      iCut = 17;
      m_oHarvestData.SetCutType(iCut);
      fail("SetCutType did not catch a cut type value of " +
           String.valueOf(iCut));
    }
    catch (ModelException oErr) {
      System.out.println("Exception processing succeeded for SetCutType.");
    }
  }

  public void testGetSetTimestep() {
    int iTimestep, iActual;

    try {
      iTimestep = 0;
      m_oHarvestData.SetTimestep(iTimestep);
      iActual = m_oHarvestData.GetTimestep();
      assertEquals(iActual, iTimestep);

      iTimestep = 10;
      m_oHarvestData.SetTimestep(iTimestep);
      iActual = m_oHarvestData.GetTimestep();
      assertEquals(iActual, iTimestep);

    }
    catch (ModelException oErr) {
      fail("Normal processing for SetCutTimestep failed with message " +
           oErr.sMessage);
    }

    try {
      iTimestep = -3;
      m_oHarvestData.SetTimestep(iTimestep);
      fail("SetTimestep did not catch a timestep value of " +
           String.valueOf(iTimestep));
    }
    catch (ModelException oErr) {
      System.out.println("Exception processing succeeded for SetTimestep.");
    }

  }

  public void testAddSpecies() {
    int iSpecies, iActual;

    //No value has been set
    iSpecies = 0;
    iActual = m_oHarvestData.GetNumberOfSpecies();
    assertEquals(iActual, iSpecies);

    //No value has been set - attempting to get species out
    try {
      iSpecies = 1;
      iActual = m_oHarvestData.GetSpecies(iSpecies);
      fail("GetSpecies did not throw an error when there were no species.");
    }
    catch (ModelException oErr) {
      ;
    }

    try {
      //Add some values
      iSpecies = 0;
      m_oHarvestData.AddSpecies(iSpecies);
      iActual = m_oHarvestData.GetNumberOfSpecies();
      assertEquals(iActual, 1);
      iActual = m_oHarvestData.GetSpecies(0);
      assertEquals(iActual, iSpecies);

      iSpecies = 2;
      m_oHarvestData.AddSpecies(iSpecies);
      iActual = m_oHarvestData.GetNumberOfSpecies();
      assertEquals(iActual, 2);
      iActual = m_oHarvestData.GetSpecies(1);
      assertEquals(iActual, iSpecies);

      //Adding duplicate
      m_oHarvestData.AddSpecies(iSpecies);
      iActual = m_oHarvestData.GetNumberOfSpecies();
      assertEquals(iActual, 2);
      iActual = m_oHarvestData.GetSpecies(1);
      assertEquals(iActual, iSpecies);

    }
    catch (ModelException oErr) {
      fail("Normal processing for AddSpecies, GetNumberOfSpecies, and GetSpecies failed with message " +
           oErr.sMessage);
    }

    //Exception processing
    try {
      iActual = m_oHarvestData.GetSpecies(3);
      fail(
          "GetSpecies did not throw an error when the species index was too large.");
    }
    catch (ModelException oErr) {
      ;
    }

    try {
      iActual = m_oHarvestData.GetSpecies( -3);
      fail(
          "GetSpecies did not throw an error when the species index was negative.");
    }
    catch (ModelException oErr) {
      ;
    }

    System.out.println(
        "Normal processing succeeded for species getting and setting.");
    System.out.println(
        "Exception processing succeeded for species getting and setting.");
  }

  public void testRemoveSpecies() {
    try {
      //Load up a species list
      HarvestData oData = new HarvestData();
      oData.AddSpecies(4);
      oData.AddSpecies(3);
      oData.AddSpecies(2);
      oData.AddSpecies(1);

      assertEquals(4, oData.GetNumberOfSpecies());

      //Try to remove a species that doesn't exist
      oData.RemoveSpecies(5);
      assertEquals(4, oData.GetNumberOfSpecies());

      //Remove one in the middle
      oData.RemoveSpecies(3);
      assertEquals(3, oData.GetNumberOfSpecies());
      assertEquals(4, oData.GetSpecies(0));
      assertEquals(2, oData.GetSpecies(1));
      assertEquals(1, oData.GetSpecies(2));

      //Remove the first one
      oData.RemoveSpecies(4);
      assertEquals(2, oData.GetNumberOfSpecies());
      assertEquals(2, oData.GetSpecies(0));
      assertEquals(1, oData.GetSpecies(1));

      //Remove the last one
      oData.RemoveSpecies(1);
      assertEquals(1, oData.GetNumberOfSpecies());
      assertEquals(2, oData.GetSpecies(0));

      //Remove the only one
      oData.RemoveSpecies(2);
      assertEquals(0, oData.GetNumberOfSpecies());

    }
    catch (ModelException oErr) {
      fail("Species removal testing failed with message " + oErr.getMessage());
    }

    System.out.println(
        "Normal processing succeeded for species removal.");
  }

  public void testCutRanges() {
    float fLow = 0, fHigh = 0, fAmountToCut = 0;
    int iIndex;

    //Normal processing
    try {

      //No cut ranges
      iIndex = m_oHarvestData.GetNumberOfCutRanges();
      assertEquals(iIndex, 0);

      //First cut range setting
      fLow = 2;
      fHigh = 5;
      fAmountToCut = (float) 45.2;
      m_oHarvestData.AddCutRange(fLow, fHigh, fAmountToCut);
      iIndex = m_oHarvestData.GetNumberOfCutRanges();
      assertEquals(iIndex, 1);

      //Second cut range
      fLow = (float) 6.5;
      fHigh = 10;
      fAmountToCut = (float) 35.2;
      m_oHarvestData.AddCutRange(fLow, fHigh, fAmountToCut);

      //Retrieving cut range values
      assertEquals(fLow, m_oHarvestData.GetLowerBound(1), 0);
      assertEquals(fHigh, m_oHarvestData.GetUpperBound(1), 0);
      assertEquals(fAmountToCut, m_oHarvestData.GetCutAmount(1), 0);

      fLow = 2;
      fHigh = 5;
      fAmountToCut = (float) 45.2;
      assertEquals(fLow, m_oHarvestData.GetLowerBound(0), 0);
      assertEquals(fHigh, m_oHarvestData.GetUpperBound(0), 0);
      assertEquals(fAmountToCut, m_oHarvestData.GetCutAmount(0), 0);

      System.out.println("Normal processing succeeded for cut ranges.");

    }
    catch (ModelException oErr) {
      fail("Normal processing for cut ranges failed with message " +
           oErr.sMessage);
    }

    //Exception processing

    //Negative lower bound
    try {
      fLow = -23;
      fHigh = 4;
      fAmountToCut = (float) 56.32;
      m_oHarvestData.AddCutRange(fLow, fHigh, fAmountToCut);
      fail("AddCutRange did not reject negative lower bound.");
    }
    catch (ModelException oErr) {
      ;
    }

    //Negative upper bound
    try {
      fLow = (float) 2.4;
      fHigh = -2;
      m_oHarvestData.AddCutRange(fLow, fHigh, fAmountToCut);
      fail("AddCutRange did not reject negative upper bound.");
    }
    catch (ModelException oErr) {
      ;
    }

    //Negative cut amount
    try {
      fLow = 2;
      fHigh = 4;
      fAmountToCut = (float) - 56.32;
      m_oHarvestData.AddCutRange(fLow, fHigh, fAmountToCut);
      fail("AddCutRange did not reject negative cut amount.");
    }
    catch (ModelException oErr) {
      ;
    }

    //Lower bound less than upper bound
    try {
      fLow = 6;
      fHigh = 4;
      fAmountToCut = (float) 56.32;
      m_oHarvestData.AddCutRange(fLow, fHigh, fAmountToCut);
      fail("AddCutRange did not reject lower bound greater than upper bound.");
    }
    catch (ModelException oErr) {
      ;
    }

    //Non-percentage value
    try {
      fLow = 3;
      fHigh = 4;
      fAmountToCut = (float) 145;
      m_oHarvestData.SetCutAmountType(DisturbanceBehaviors.
                                      PERCENTAGE_BASAL_AREA);
      m_oHarvestData.AddCutRange(fLow, fHigh, fAmountToCut);
      fail("AddCutRange did not reject non-percentage.");
    }
    catch (ModelException oErr) {
      ;
    }
    try {
      fLow = 3;
      fHigh = 4;
      fAmountToCut = (float) 145;
      m_oHarvestData.SetCutAmountType(DisturbanceBehaviors.PERCENTAGE_DENSITY);
      m_oHarvestData.AddCutRange(fLow, fHigh, fAmountToCut);
      fail("AddCutRange did not reject non-percentage.");
    }
    catch (ModelException oErr) {
      ;
    }

    //Invalid cut range index
    try {
      m_oHarvestData.GetLowerBound(4);
      fail("GetLowerBound did not reject bad index.");
    }
    catch (ModelException oErr) {
      ;
    }
    try {
      m_oHarvestData.GetUpperBound(4);
      fail("GetUpperBound did not reject bad index.");
    }
    catch (ModelException oErr) {
      ;
    }
    try {
      m_oHarvestData.GetCutAmount(4);
      fail("GetCutAmount did not reject bad index.");
    }
    catch (ModelException oErr) {
      ;
    }

    //Too many cut ranges
    try {
      assertEquals(2, m_oHarvestData.GetNumberOfCutRanges());

      fLow = 3;
      fHigh = 4;
      fAmountToCut = 34;
      m_oHarvestData.AddCutRange(fLow, fHigh, fAmountToCut);
      m_oHarvestData.AddCutRange(fLow, fHigh, fAmountToCut);
      m_oHarvestData.AddCutRange(fLow, fHigh, fAmountToCut);
    }
    catch (ModelException oErr) {
      ;
    }

    System.out.println(
        "Exception processing succeeded for cut ranges getting and setting.");
  }

  public void testCells() {
    Plot oPlot = null;
    //Normal processing
    try {
      oPlot = new Plot(null);
      oPlot.SetPlotXLength(100);
      oPlot.SetPlotYLength(100);

      assertEquals(m_oHarvestData.GetNumberOfCells(), 0);

      m_oHarvestData.AddCell(0, 0, oPlot);
      assertEquals(m_oHarvestData.GetNumberOfCells(), 1);
      assertEquals(0, m_oHarvestData.GetCell(0).GetX());
      assertEquals(0, m_oHarvestData.GetCell(0).GetY());

      m_oHarvestData.AddCell(9, 9, oPlot);
      assertEquals(m_oHarvestData.GetNumberOfCells(), 2);
      assertEquals(9, m_oHarvestData.GetCell(1).GetX());
      assertEquals(9, m_oHarvestData.GetCell(1).GetY());

      m_oHarvestData.AddCell(3, 4, oPlot);
      assertEquals(m_oHarvestData.GetNumberOfCells(), 3);
      assertEquals(3, m_oHarvestData.GetCell(2).GetX());
      assertEquals(4, m_oHarvestData.GetCell(2).GetY());

      //Duplicate cell
      m_oHarvestData.AddCell(9, 9, oPlot);
      assertEquals(m_oHarvestData.GetNumberOfCells(), 3);

      //New cell passed as cell
      Cell oCell = new Cell(2, 6, oPlot);
      m_oHarvestData.AddCell(oCell);
      assertEquals(4, m_oHarvestData.GetNumberOfCells());
      assertEquals(2, m_oHarvestData.GetCell(3).GetX());
      assertEquals(6, m_oHarvestData.GetCell(3).GetY());

      //Duplicate passed as cell
      oCell = new Cell(3, 4, oPlot);
      m_oHarvestData.AddCell(oCell);
      assertEquals(4, m_oHarvestData.GetNumberOfCells());

    }
    catch (ModelException oErr) {
      System.out.println("Normal processing failed for cells with message " +
                         oErr.sMessage);
    }

    System.out.println("Normal processing succeeded for cells.");

    //Exception processing
    try {
      //Bad cell index
      m_oHarvestData.GetCell(6);
      fail("GetCell did not reject bad cell index.");
    }
    catch (ModelException oErr) {
      ;
    }
    try {
      //Bad cell index
      m_oHarvestData.GetCell( -6);
      fail("GetCell did not reject bad cell index.");
    }
    catch (ModelException oErr) {
      ;
    }
    System.out.println("Exception processing succeeded for cells.");
  }

  public void testValidation() {
    HarvestData oToTest = null;
    Plot oPlot = null;
    TreePopulation oPop = null;

    try {
      oPlot = new Plot(null);
      oPlot.SetPlotXLength(100);
      oPlot.SetPlotYLength(100);

      oPop = new TreePopulation(null);
      oPop.SetSpeciesNames(new String[] {"ACRU", "ACSA"});

      //Validation passes
      oToTest = new HarvestData();
      oToTest.SetCutAmountType(DisturbanceBehaviors.PERCENTAGE_BASAL_AREA);
      oToTest.SetCutType(DisturbanceBehaviors.CLEAR_CUT);
      oToTest.SetTimestep(1);
      oToTest.AddCell(0, 0, oPlot);
      oToTest.AddSpecies(0);
      oToTest.AddCutRange(1, 2, 3);
      oToTest.ValidateCut(oPop, oPlot);

    }
    catch (ModelException oErr) {
      System.out.println(
          "Normal processing failed for validation with message " +
          oErr.sMessage);
    }

    //Exception processing
    //No species
    try {
      oToTest = null;
      oToTest = new HarvestData();
      oToTest.SetCutAmountType(DisturbanceBehaviors.PERCENTAGE_BASAL_AREA);
      oToTest.SetCutType(DisturbanceBehaviors.CLEAR_CUT);
      oToTest.SetTimestep(1);
      oToTest.AddCell(0, 0, oPlot);
      oToTest.AddCutRange(1, 2, 3);
      oToTest.ValidateCut(oPop, oPlot);
      fail("Validation did not catch lack of species.");
    }
    catch (ModelException oErr) {
      ;
    }

    //Invalid species
    try {
      oToTest = null;
      oToTest = new HarvestData();
      oToTest.SetCutAmountType(DisturbanceBehaviors.PERCENTAGE_BASAL_AREA);
      oToTest.SetCutType(DisturbanceBehaviors.CLEAR_CUT);
      oToTest.SetTimestep(1);
      oToTest.AddCell(0, 0, oPlot);
      oToTest.AddCutRange(1, 2, 3);
      oToTest.AddSpecies(0);
      oToTest.AddSpecies(2);
      oToTest.ValidateCut(oPop, oPlot);
      fail("Validation did not catch invalid species.");
    }
    catch (ModelException oErr) {
      ;
    }

    //No cut amount type specified
    try {
      oToTest = null;
      oToTest = new HarvestData();
      oToTest.SetCutType(DisturbanceBehaviors.CLEAR_CUT);
      oToTest.SetTimestep(1);
      oToTest.AddCell(0, 0, oPlot);
      oToTest.AddCutRange(1, 2, 3);
      oToTest.AddSpecies(0);
      oToTest.ValidateCut(oPop, oPlot);
      fail("Validation did not catch lack of cut amount type.");
    }
    catch (ModelException oErr) {
      ;
    }

    //No cut type specified
    try {
      oToTest = null;
      oToTest = new HarvestData();
      oToTest.SetCutAmountType(DisturbanceBehaviors.PERCENTAGE_BASAL_AREA);
      oToTest.SetTimestep(1);
      oToTest.AddCell(0, 0, oPlot);
      oToTest.AddCutRange(1, 2, 3);
      oToTest.AddSpecies(0);
      oToTest.ValidateCut(oPop, oPlot);
      fail("Validation did not catch lack of cut type.");
    }
    catch (ModelException oErr) {
      ;
    }

    //No timestep specified
    try {
      oToTest = null;
      oToTest = new HarvestData();
      oToTest.SetCutAmountType(DisturbanceBehaviors.PERCENTAGE_BASAL_AREA);
      oToTest.SetCutType(DisturbanceBehaviors.CLEAR_CUT);
      oToTest.AddCell(0, 0, oPlot);
      oToTest.AddCutRange(1, 2, 3);
      oToTest.AddSpecies(0);
      oToTest.ValidateCut(oPop, oPlot);
      fail("Validation did not catch lack of timesteps.");
    }
    catch (ModelException oErr) {
      ;
    }

    //No cut range specified
    try {
      oToTest = null;
      oToTest = new HarvestData();
      oToTest.SetCutAmountType(DisturbanceBehaviors.PERCENTAGE_BASAL_AREA);
      oToTest.SetCutType(DisturbanceBehaviors.CLEAR_CUT);
      oToTest.SetTimestep(1);
      oToTest.AddCell(0, 0, oPlot);
      oToTest.AddSpecies(0);
      oToTest.ValidateCut(oPop, oPlot);
      fail("Validation did not catch lack of cut ranges.");
    }
    catch (ModelException oErr) {
      ;
    }

    //No cells
    try {
      oToTest = null;
      oToTest = new HarvestData();
      oToTest.SetCutAmountType(DisturbanceBehaviors.PERCENTAGE_BASAL_AREA);
      oToTest.SetCutType(DisturbanceBehaviors.CLEAR_CUT);
      oToTest.SetTimestep(1);
      oToTest.AddCutRange(1, 2, 3);
      oToTest.AddSpecies(0);
      oToTest.ValidateCut(oPop, oPlot);
      fail("Validation did not catch lack of cells.");
    }
    catch (ModelException oErr) {
      ;
    }
  }
}
